@@ -51,8 +51,6 @@ extern "C" {
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/stat.h>
-#include <sys/types.h>
 #include <time.h>
 #include <math.h>
 
@@ -261,25 +259,64 @@ extern int SystemImpl__setLDFlags(const char *str)
   return 0;
 }
 
+#if defined(__MINGW32__) || defined(_MSC_VER)
+/* Make sure windows paths use frontslash and not backslash */
+void SystemImpl__toWindowsSeperators(char* buffer, int bufferLength)
+{
+  int i;
+  for (i=0; i<bufferLength && buffer[i]; i++) {
+    if (buffer[i] == '\\') buffer[i] = '/';
+  }
+}
+#endif
+
+int SystemImpl__chdir(const char* path)
+{
+#if defined(__MINGW32__) || defined(_MSC_VER)
+  MULTIBYTE_TO_WIDECHAR_LENGTH(path, unicodePathLength);
+  MULTIBYTE_TO_WIDECHAR_VAR(path, unicodePath, unicodePathLength);
+
+  if (!SetCurrentDirectoryW(unicodePath)) {
+    MULTIBYTE_OR_WIDECHAR_VAR_FREE(unicodePath);
+    c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("SetCurrentDirectoryW failed."),NULL,0);
+    return -1;
+  }
+  MULTIBYTE_OR_WIDECHAR_VAR_FREE(unicodePath);
+  return 0;
+#else
+  if (chdir(path) != 0) {
+    c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("chdir failed."),NULL,0);
+    return -1;
+  }
+  return 0;
+#endif
+}
+
 extern char* SystemImpl__pwd(void)
 {
-  char buf[MAXPATHLEN];
 #if defined(__MINGW32__) || defined(_MSC_VER)
-  int i;
-  LPTSTR bufPtr=buf;
-  DWORD bufLen = MAXPATHLEN;
-  if (!GetCurrentDirectory(bufLen,bufPtr)) {
-    fprintf(stderr, "System.pwd failed\n");
+  DWORD bufLen = 0;
+  bufLen = GetCurrentDirectoryW(bufLen, 0);
+  if (!bufLen) {
+    c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("GetCurrentDirectoryW failed."),NULL,0);
     return NULL;
   }
-  /* Make sure windows paths use frontslash and not backslash */
-  for (i=0; i<MAXPATHLEN && buf[i]; i++) {
-    if (buf[i] == '\\') buf[i] = '/';
+
+  WCHAR unicodePath[bufLen];
+  if (!GetCurrentDirectoryW(bufLen, unicodePath)) {
+    c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("GetCurrentDirectoryW failed."),NULL,0);
+    return NULL;
   }
-  return omc_alloc_interface.malloc_strdup(buf);
+  WIDECHAR_TO_MULTIBYTE_LENGTH(unicodePath, bufferLength);
+  WIDECHAR_TO_MULTIBYTE_VAR(unicodePath, buffer, bufferLength);
+  SystemImpl__toWindowsSeperators(buffer, bufferLength);
+  char *res = omc_alloc_interface.malloc_strdup(buffer);
+  MULTIBYTE_OR_WIDECHAR_VAR_FREE(buffer);
+  return res;
 #else
+  char buf[MAXPATHLEN];
   if (NULL == getcwd(buf,MAXPATHLEN)) {
-    fprintf(stderr, "System.pwd failed\n");
+    c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("System.pwd failed."),NULL,0);
     return NULL;
   }
   return omc_alloc_interface.malloc_strdup(buf);
@@ -288,11 +325,16 @@ extern char* SystemImpl__pwd(void)
 
 extern int SystemImpl__regularFileExists(const char* str)
 {
-#if defined(_MSC_VER)
-  WIN32_FIND_DATA FileData;
+#if defined(__MINGW32__) || defined(_MSC_VER)
+  WIN32_FIND_DATAW FileData;
   HANDLE sh;
 
-  sh = FindFirstFile(str, &FileData);
+  MULTIBYTE_TO_WIDECHAR_LENGTH(str, unicodeFilenameLength);
+  MULTIBYTE_TO_WIDECHAR_VAR(str, unicodeFilename, unicodeFilenameLength);
+
+  sh = FindFirstFileW(unicodeFilename, &FileData);
+
+  MULTIBYTE_OR_WIDECHAR_VAR_FREE(unicodeFilename);
 
   if (sh == INVALID_HANDLE_VALUE) {
     if (strlen(str) >= MAXPATHLEN)
@@ -322,7 +364,7 @@ extern int SystemImpl__regularFileWritable(const char* str)
   FILE *f;
   if (!SystemImpl__regularFileExists(str))
     return 0;
-  f = fopen(str, "a");
+  f = omc_fopen(str, "a");
   if (f == NULL)
     return 0;
   fclose(f);
@@ -334,8 +376,12 @@ static char* SystemImpl__readFile(const char* filename)
   char* buf;
   int res;
   FILE * file = NULL;
+#if defined(__MINGW32__) || defined(_MSC_VER)
+  struct _stat statstr;
+#else
   struct stat statstr;
-  res = stat(filename, &statstr);
+#endif
+  res = omc_stat(filename, &statstr);
 
   if (res != 0) {
     const char *c_tokens[2]={strerror(errno),filename};
@@ -362,7 +408,7 @@ static char* SystemImpl__readFile(const char* filename)
   }
 #endif
 
-  file = fopen(filename,"rb");
+  file = omc_fopen(filename,"rb");
   if (file == NULL) {
     const char *c_tokens[2]={strerror(errno),filename};
     c_add_message(NULL, 85, /* ERROR_OPENING_FILE */
@@ -394,11 +440,7 @@ static char* SystemImpl__readFile(const char* filename)
 /* returns 0 on success */
 int SystemImpl__removeFile(const char* filename)
 {
-#if defined(__MINGW32__) || defined(_MSC_VER)
-  return _unlink(filename);
-#else /* unix */
-  return unlink(filename);
-#endif
+  return omc_unlink(filename);
 }
 
 /* returns 0 on success */
@@ -412,10 +454,10 @@ int SystemImpl__writeFile(const char* filename, const char* data)
   FILE * file = NULL;
   int len = strlen(data); /* MMC_HDRSTRLEN(MMC_GETHDR(rmlA1)); */
 #if defined(__APPLE_CC__)||defined(__MINGW32__)||defined(__MINGW64__)
-  unlink(filename);
+  SystemImpl__removeFile(filename);
 #endif
   /* adrpo: 2010-09-22 open the file in BINARY mode as otherwise \r\n becomes \r\r\n! */
-  file = fopen(filename,fileOpenMode);
+  file = omc_fopen(filename,fileOpenMode);
   if (file == NULL) {
     const char *c_tokens[1]={filename};
     c_add_message(NULL,21, /* WRITING_FILE_ERROR */
@@ -545,7 +587,7 @@ const char* SystemImpl__basename(const char *str)
 #if defined(__MINGW32__) || defined(_MSC_VER)
 int runProcess(const char* cmd)
 {
-  STARTUPINFO si;
+  STARTUPINFOW si;
   PROCESS_INFORMATION pi;
   char *c = "cmd /c";
   char *command = (char *)omc_alloc_interface.malloc_atomic(strlen(cmd) + strlen(c) + 4);
@@ -560,14 +602,18 @@ int runProcess(const char* cmd)
 
   /* fprintf(stderr, "%s\n", command); fflush(NULL); */
 
-  if (CreateProcessA(NULL, command, NULL, NULL, FALSE, CREATE_NO_WINDOW, NULL, NULL, &si, &pi))
+  MULTIBYTE_TO_WIDECHAR_LENGTH(command, unicodeCommandLength);
+  MULTIBYTE_TO_WIDECHAR_VAR(command, unicodeCommand, unicodeCommandLength);
+
+  if (CreateProcessW(NULL, unicodeCommand, NULL, NULL, FALSE, CREATE_NO_WINDOW, NULL, NULL, &si, &pi))
   {
-      WaitForSingleObject(pi.hProcess, INFINITE);
-      // Get the exit code.
-      GetExitCodeProcess(pi.hProcess, &exitCode);
-      CloseHandle(pi.hProcess);
-      CloseHandle(pi.hThread);
+    WaitForSingleObject(pi.hProcess, INFINITE);
+    // Get the exit code.
+    GetExitCodeProcess(pi.hProcess, &exitCode);
+    CloseHandle(pi.hProcess);
+    CloseHandle(pi.hThread);
   }
+  MULTIBYTE_OR_WIDECHAR_VAR_FREE(unicodeCommand);
   GC_free(command);
   return (int)exitCode;
 }
@@ -695,13 +741,13 @@ char* System_popen(threadData_t *threadData, const char* command, int *status)
 #endif
 }
 
-#ifdef WITH_HWLOC
+#if WITH_HWLOC==1
 #include <hwloc.h>
 #endif
 
 int System_numProcessors(void)
 {
-#ifdef WITH_HWLOC
+#if WITH_HWLOC==1
   hwloc_topology_t topology;
   if (0==hwloc_topology_init(&topology) && 0==hwloc_topology_load(topology)) {
     int depth = hwloc_get_type_depth(topology, HWLOC_OBJ_CORE);
@@ -896,27 +942,52 @@ extern int SystemImpl__createDirectory(const char *str)
 extern int SystemImpl__copyFile(const char *str_1, const char *str_2)
 {
   int rv = 1;
-  char ch;
+  size_t n;
+  char buf[8192];
   FILE *source, *target;
 
-  if (!SystemImpl__directoryExists(str_2))
-  {
-      rv = SystemImpl__createDirectory(str_2);
+  source = fopen(str_1, "r");
+  if (source==0) {
+    const char *msg[2] = {strerror(errno), str_1};
+    c_add_message(NULL,85,
+      ErrorType_scripting,
+      ErrorLevel_error,
+      gettext("Error opening file for reading %s: %s"),
+      msg,
+      2);
+    return 0;
+  }
+  target = fopen(str_2, "w");
+  if (target==0) {
+    const char *msg[2] = {strerror(errno), str_2};
+    c_add_message(NULL,85,
+      ErrorType_scripting,
+      ErrorLevel_error,
+      gettext("Error opening file for writing %s: %s"),
+      msg,
+      2);
+    fclose(source);
+    return 0;
   }
 
-  if (strcmp(str_1, "") == 0)
+  while ( n = fread(buf, 1, 8192, source) ) {
+    if (n != fwrite(buf, 1, n, target)) {
+      rv = 0;
+      break;
+    }
+  }
+  if (rv == 0) {
+    const char *msg[3] = {strerror(errno), str_2, str_1};
+    c_add_message(NULL,85,
+      ErrorType_scripting,
+      ErrorLevel_error,
+      gettext("Error copying file contents %s to %s: %s"),
+      msg,
+      3);
+  }
+  if (!feof(source)) {
     rv = 0;
-
-  char targetFile[100];
-  strcpy(targetFile,str_2);
-  strcat(targetFile,"/");
-  strcat(targetFile,str_1);
-
-  source = fopen(str_1, "r");
-  target = fopen(targetFile, "w");
-
-  while( ( ch = fgetc(source) ) != EOF )
-     rv=rv && fputc(ch, target);
+  }
 
   fclose(source);
   fclose(target);
@@ -1150,8 +1221,12 @@ extern const char* SystemImpl__readFileNoNumeric(const char* filename)
   char* buf, *bufRes;
   int res,numCount;
   FILE * file = NULL;
+#if defined(__MINGW32__) || defined(_MSC_VER)
+  struct _stat statstr;
+#else
   struct stat statstr;
-  res = stat(filename, &statstr);
+#endif
+  res = omc_stat(filename, &statstr);
 
   if(res!=0) {
     const char *c_tokens[1]={filename};
@@ -1164,7 +1239,7 @@ extern const char* SystemImpl__readFileNoNumeric(const char* filename)
     return "No such file";
   }
 
-  file = fopen(filename,"rb");
+  file = omc_fopen(filename,"rb");
   buf = (char*) omc_alloc_interface.malloc_atomic(statstr.st_size+1);
   bufRes = (char*) omc_alloc_interface.malloc_atomic((statstr.st_size+70)*sizeof(char));
   if( (res = fread(buf, sizeof(char), statstr.st_size, file)) != statstr.st_size) {
@@ -1645,29 +1720,124 @@ extern char* SystemImpl__unescapedString(const char* str)
   return res;
 }
 
-extern char* SystemImpl__unquoteIdentifier(const char* str)
-{
-  const char lookupTbl[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
-  char *res,*cur;
-  int len,i;
-  const int offset = 10;
-  const char _omcQuot[]="_omcQuot_";
-  if (*str != '\'') return NULL;
-  len = strlen(str)-2;
-  res = (char*) omc_alloc_interface.malloc_atomic(2*len+offset+64);
-  cur = res;
-  cur += sprintf(cur,"%s",_omcQuot);
-  for (i=0; i<len; i++) {
-    unsigned char c = str[i+1];
-    *cur = lookupTbl[c/16];
-    cur++;
-    *cur = lookupTbl[c%16];
-    cur++;
+
+static int compute_sanitized_string_size(const char* str) {
+  int i, count;
+
+  for (i=0, count=0; str[i]; i++, count++) {
+    char c = str[i];
+    // Each non-alphanum character needs one more char for its
+    // two char ascii representation.
+    if (!isalnum(c) && c != '_') {
+      count++;
+    }
   }
+
+  return count;
+}
+
+static char* sanitize_string(const char* src, char* dst, int nrchars) {
+  const char lookupTbl[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
+
+  int i;
+  for (i = 0; i < nrchars; i++) {
+    unsigned char c = src[i];
+    if (isalnum(c) || c == '_') {
+      *dst = c;
+      dst++;
+    }
+    else {
+      *dst = lookupTbl[c/16];
+      dst++;
+      *dst = lookupTbl[c%16];
+      dst++;
+    }
+  }
+
+  return dst;
+}
+
+// This function assumes the input is actually a quoted string e.g 'gb' or 'ab[!%'
+extern char* System_sanitizeQuotedIdentifier(const char* str)
+{
+  char *res,*cur;
+
+  const char openquote[]="Q_";
+  const char closequote[]="_Q";
+  const int qsize = sizeof(openquote) - 1;
+
+  int nrchars_org = strlen(str);
+
+  // Each non-alphanum character needs one more char for its
+  // two char ascii representation.
+  int nrchars_needed = compute_sanitized_string_size(str);
+
+  // ignore the count for opening and closing quotes in the original string
+  nrchars_needed = nrchars_needed - 4;
+  nrchars_org = nrchars_org - 2;
+
+  nrchars_needed += 2*qsize; // opening and closing quoute rep.
+  nrchars_needed++; /*null terminator*/
+  res = (char*) omc_alloc_interface.malloc_atomic(nrchars_needed * sizeof(char));
+
+  cur = res;
+  cur += sprintf(cur, "%s", openquote);
+  cur = sanitize_string(str + 1, cur, nrchars_org);
+  cur += sprintf(cur,"%s", closequote);
   *cur = '\0';
+
+  ++cur;
+  assert((cur == res + nrchars_needed) && "Allocated memory does not exactly fit the unquoted string output");
+
   return res;
 }
 
+extern char* System_sanitizeIdentifier(const char* str)
+{
+  char *res,*cur;
+
+  int nrchars_org = strlen(str);
+
+  // Each non-alphanum character needs one more char for its
+  // two char ascii representation.
+  int nrchars_needed = compute_sanitized_string_size(str);
+
+  // if the first char is not alphanum then the result will start with a number(ascii)
+  // we do not want that. So make it always start with a char.
+  const char openquote[]="D_";
+  const int qsize = sizeof(openquote) - 1;
+  nrchars_needed += qsize;
+
+  nrchars_needed++; /*null terminator*/
+  res = (char*) omc_alloc_interface.malloc_atomic(nrchars_needed * sizeof(char));
+
+  cur = res;
+  cur += sprintf(cur, "%s", openquote);
+  cur = sanitize_string(str, cur, nrchars_org);
+  *cur = '\0';
+
+  ++cur;
+  assert((cur == res + nrchars_needed) && "Allocated memory does not exactly fit the unquoted string output");
+
+  return res;
+}
+
+extern char* SystemImpl__unquoteIdentifier(char* str)
+{
+
+  if (str[0] == '\'') {
+    return System_sanitizeQuotedIdentifier(str);
+  }
+
+#if !defined(OPENMODELICA_BOOTSTRAPPING_STAGE_1)
+  if (strstr(str, "$")) {
+    return System_sanitizeIdentifier(str);
+  }
+#endif
+
+  return str;
+}
+
 #define TIMER_MAX_STACK  1000
 static double timerIntervalTime = 0;
 static double timerCummulatedTime = 0;
@@ -1999,7 +2169,7 @@ static modelicaPathEntry* getAllModelicaPaths(const char *name, size_t nlen, voi
           /* fprintf(stderr, "found dir match: %ld %s - ok=%d\n", i, ent->d_name, ok); */
         }
         entlen = strlen(ent->d_name);
-        if (!ok && ((entlen > 3 && 0==strcmp(ent->d_name+entlen-3,".mo")) || entlen > 4 && 0==strcmp(ent->d_name+entlen-4,".moc")) && regularFileExistsInDirectory(mp,"",ent->d_name)) {
+        if (!ok && ((entlen > 3 && 0==strcmp(ent->d_name+entlen-3,".mo")) || (entlen > 4 && 0==strcmp(ent->d_name+entlen-4,".moc"))) && regularFileExistsInDirectory(mp,"",ent->d_name)) {
           /* fprintf(stderr, "found match file: %ld %s - ok=%d\n", i, ent->d_name, ok); */
           res[i].fileIsDir=0;
           ok=1;
@@ -2311,7 +2481,7 @@ extern const char* SystemImpl__iconv(const char * str, const char *from, const c
     if (printError) {
       const char *ignore = SystemImpl__iconv__ascii(str);
       const char *tokens[4] = {res,from,to,ignore};
-      c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("iconv(\"%s\",to=\"%s\",from=\"%s\") failed: %s"),tokens,4);
+      c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("iconv(\"%s\",from=\"%s\",to=\"%s\") failed: %s"),tokens,4);
       omc_alloc_interface.free_uncollectable((char*)ignore);
     }
     return (const char*) "";
@@ -2510,9 +2680,21 @@ int System_getTerminalWidth(void)
 
 #include "simulation_options.h"
 
-#define SB_SIZE 8192*2
+#define SB_SIZE 8192*4
 #define SB_SIZE_MINUS_ONE (SB_SIZE-1)
 
+/* snprintf check negative size */
+static size_t check_nonnegative(long sz)
+{
+  if (sz < 0) {
+    fprintf(stderr, "%s:%d got negative size: %ld, which should not happen\n", __FILE__, __LINE__, sz);
+    exit(EXIT_FAILURE);
+  }
+  return sz;
+}
+
+#define CHECK_NONNEGATIVE_BUFFER() check_nonnegative((long)SB_SIZE_MINUS_ONE-(cur-buf))
+
 char* System_getSimulationHelpTextSphinx(int detailed, int sphinx)
 {
   static char buf[SB_SIZE];
@@ -2523,13 +2705,13 @@ char* System_getSimulationHelpTextSphinx(int detailed, int sphinx)
   for(i=1; i<FLAG_MAX; ++i)
   {
     if (sphinx) {
-      cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), "\n.. _simflag-%s :\n\n", FLAG_NAME[i]);
+      cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), "\n.. _simflag-%s :\n\n", FLAG_NAME[i]);
     }
     if (FLAG_TYPE[i] == FLAG_TYPE_FLAG) {
       if (sphinx) {
-        cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), ":ref:`-%s <simflag-%s>`\n%s\n", FLAG_NAME[i], FLAG_NAME[i], desc[i]);
+        cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), ":ref:`-%s <simflag-%s>`\n%s\n", FLAG_NAME[i], FLAG_NAME[i], desc[i]);
       } else {
-        cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), "<-%s>\n%s\n", FLAG_NAME[i], desc[i]);
+        cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), "<-%s>\n%s\n", FLAG_NAME[i], desc[i]);
       }
     } else if (FLAG_TYPE[i] == FLAG_TYPE_OPTION) {
       int numExtraFlags=0;
@@ -2537,9 +2719,9 @@ char* System_getSimulationHelpTextSphinx(int detailed, int sphinx)
       const char **flagName;
       const char **flagDesc;
       if (sphinx) {
-        cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), ":ref:`-%s=value <simflag-%s>` *or* -%s value \n%s\n", FLAG_NAME[i], FLAG_NAME[i], FLAG_NAME[i], desc[i]);
+        cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), ":ref:`-%s=value <simflag-%s>` *or* -%s value \n%s\n", FLAG_NAME[i], FLAG_NAME[i], FLAG_NAME[i], desc[i]);
       } else {
-        cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), "<-%s=value> or <-%s value>\n%s\n", FLAG_NAME[i], FLAG_NAME[i], desc[i]);
+        cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), "<-%s=value> or <-%s value>\n%s\n", FLAG_NAME[i], FLAG_NAME[i], desc[i]);
       }
 
       switch(i) {
@@ -2608,20 +2790,20 @@ char* System_getSimulationHelpTextSphinx(int detailed, int sphinx)
       }
 
       if (numExtraFlags) {
-        cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), "\n");
+        cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), "\n");
         if (flagName) {
           for (j=firstExtraFlag; j<numExtraFlags; j++) {
-            cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), "  * %s (%s)\n", flagName[j], flagDesc[j]);
+            cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), "  * %s (%s)\n", flagName[j], flagDesc[j]);
           }
         } else {
           for (j=firstExtraFlag; j<numExtraFlags; j++) {
-            cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), "  * %s\n", flagDesc[j]);
+            cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), "  * %s\n", flagDesc[j]);
           }
         }
       }
 
     } else {
-      cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), "[unknown flag-type] <-%s>\n", FLAG_NAME[i]);
+      cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), "[unknown flag-type] <-%s>\n", FLAG_NAME[i]);
     }
   }
   *cur = 0;
@@ -2983,11 +3165,6 @@ const char* SystemImpl__createTemporaryDirectory(const char *templatePrefix)
   MMC_THROW();
 }
 
-const char* System_getTriple()
-{
-  return DEFAULT_TRIPLE;
-}
-
 #if defined(OMC_GENERATE_RELOCATABLE_CODE)
 static void addDLError(const char *msg, const char *fileName)
 {
