@@ -45,6 +45,7 @@
 
 #include "errorext.h"
 #include "systemimpl.h"
+#include "omc_file.h"
 
 pthread_once_t parser_once_create_key = PTHREAD_ONCE_INIT;
 pthread_key_t modelicaParserKey;
@@ -180,7 +181,6 @@ static void handleLexerError(pANTLR3_BASE_RECOGNIZER recognizer, pANTLR3_UINT8 *
 static void handleParseError(pANTLR3_BASE_RECOGNIZER recognizer, pANTLR3_UINT8 * tokenNames)
 {
   pANTLR3_PARSER      parser;
-  pANTLR3_STRING      ttext;
   pANTLR3_EXCEPTION      ex;
   pANTLR3_COMMON_TOKEN   preToken,nextToken;
   pANTLR3_TOKEN_STREAM tokenStream;
@@ -195,7 +195,6 @@ static void handleParseError(pANTLR3_BASE_RECOGNIZER recognizer, pANTLR3_UINT8 *
 
   // Retrieve some info for easy reading.
   ex      =    recognizer->state->exception;
-  ttext   =    (pANTLR3_STRING) NULL;
 
   switch  (recognizer->type)
   {
@@ -383,7 +382,7 @@ static void* parseString(const char* data, const char* interactiveFilename, int
 #elif defined(ANTLR_C_VERSION_3_4)
   input  = antlr3StringStreamNew((pANTLR3_UINT8)data, ANTLR3_ENC_8BIT, strlen(data), fName);
 #else
-  #error "Neiter ANTLR_C_VERSION_3_2 or ANTLR_C_VERSION_3_4 is defined. Could not find the ANTLR 3.x C runtime!"
+  #error "Neither ANTLR_C_VERSION_3_2 or ANTLR_C_VERSION_3_4 is defined. Could not find the ANTLR 3.x C runtime!"
 #endif
   if ( input == NULL ) {
     fprintf(stderr, "Unable to open file %s\n", members.filename_C); fflush(stderr);
@@ -435,8 +434,12 @@ static void* parseFile(const char* fileName, const char* infoName, int flags, co
    * Workaround: ANTLR3 does not like 0-length files on Windows!
    * So we pass an empty string instead :)
    */
+#if defined(__MINGW32__) || defined(_MSC_VER)
+  struct _stat st;
+#else
   struct stat st;
-  stat(members.filename_C, &st);
+#endif
+  omc_stat(members.filename_C, &st);
   members.timestamp = mmc_mk_rcon((double)st.st_mtime);
   if (0 == st.st_size) return parseString("",members.filename_C,ModelicaParser_flags, langStd, runningTestsuite);
 
@@ -446,7 +449,7 @@ static void* parseFile(const char* fileName, const char* infoName, int flags, co
 #elif defined(ANTLR_C_VERSION_3_4)
   input  = antlr3FileStreamNew(fName, ANTLR3_ENC_8BIT);
 #else
-  #error "Neiter ANTLR_C_VERSION_3_2 or ANTLR_C_VERSION_3_4 is defined. Could not find the ANTLR 3.x C runtime!"
+  #error "Neither ANTLR_C_VERSION_3_2 or ANTLR_C_VERSION_3_4 is defined. Could not find the ANTLR 3.x C runtime!"
 #endif
   if ( input == NULL ) {
     return NULL;
@@ -463,6 +466,21 @@ int startLibraryVendorExecutable(const char* path, void** lveInstance)
   return 0;
 }
 
+int checkLVEToolLicense(void** lveInstance, const char* packageName)
+{
+#ifdef OMENCRYPTION
+  return checkLVEToolLicenseImpl(lveInstance, packageName);
+#endif
+  return 0;
+}
+
+void checkLVEToolFeature(void** lveInstance, const char* feature)
+{
+#ifdef OMENCRYPTION
+  checkLVEToolFeatureImpl(lveInstance, feature);
+#endif
+}
+
 void stopLibraryVendorExecutable(void** lveInstance)
 {
 #ifdef OMENCRYPTION
