@@ -31,7 +31,7 @@
 #ifndef DASSL_H
 #define DASSL_H
 
-#include "simulation/solver/solver_main.h"
+#include "solver_main.h"
 
 #define DDASKR _daskr_ddaskr_
 
@@ -67,30 +67,34 @@ typedef struct DASSL_DATA{
 
   /* variables used in jacobian calculation */
   double *ysave;
-  double *ypsave;
   double *delta_hh;
   double *newdelta;
   double *stateDer;
   double *states;
 
   /* function pointer of provided functions */
-  int (*residualFunction)(double *t, double *x, double *xprime, double *cj, double *delta, int *ires, double *rpar, int* ipar);
-  int (*jacobianFunction)(double *t, double *y, double *yprime, double *deltaD, double *pd, double *cj, double *h, double *wt,
-     double *rpar, int* ipar);
+  int (*residualFunction)(double *t, double *x, double *xprime, double *cj,
+                          double *delta, int *ires, double *rpar, int* ipar);
+  int (*jacobianFunction)(double *t, double *y, double *yprime, double *deltaD,
+                          double *pd, double *cj, double *h, double *wt,
+                          double *rpar, int* ipar);
   void* zeroCrossingFunction;
+
+#ifdef USE_PARJAC
+  ANALYTIC_JACOBIAN* jacColumns;    /* thread local analytic jacobians */
+#endif
+  int allocatedParMem; /* indicated if parallel memory was allocated, 0=false, 1=true*/
 } DASSL_DATA;
 
 /* main dassl function to make a step */
-int
-dassl_step(DATA* simData, threadData_t *threadData, SOLVER_INFO* solverInfo);
+int dassl_step(DATA* simData, threadData_t *threadData, SOLVER_INFO* solverInfo);
 
 /* initial main dassl Data */
-int
-dassl_initial(DATA* simData, threadData_t *threadData, SOLVER_INFO* solverInfo, DASSL_DATA *dasslData);
+int dassl_initial(DATA* simData, threadData_t *threadData,
+                  SOLVER_INFO* solverInfo, DASSL_DATA *dasslData);
 
 /* deinitial main dassl Data */
-int
-dassl_deinitial(DASSL_DATA *dasslData);
+int dassl_deinitial(DASSL_DATA *dasslData);
 
 int printCurrentStatesVector(int logLevel, double* states, DATA* data, double time);
 int printVector(int logLevel, const char* name,  double* vec, int n, double time);
