@@ -31,6 +31,12 @@
 /*! \file ida_solver.c
  */
 
+#ifdef USE_PARJAC
+  #include <omp.h>
+  #define GC_THREADS
+  #include <gc/omc_gc.h>
+#endif
+
 #include <string.h>
 #include <setjmp.h>
 
@@ -40,6 +46,7 @@
 #include "simulation_data.h"
 
 #include "util/omc_error.h"
+#include "util/parallel_helper.h"
 #include "gc/omc_gc.h"
 
 #include "simulation/options.h"
@@ -53,9 +60,15 @@
 #include "simulation/solver/ida_solver.h"
 #include "simulation/solver/dassl.h"
 #include "simulation/solver/dae_mode.h"
+#include "simulation/solver/jacobianSymbolical.h"
 
 #ifdef WITH_SUNDIALS
 
+/* adrpo: on mingw link with static sundials */
+#if defined(__MINGW32__)
+#define LINK_SUNDIALS_STATIC
+#endif
+
 /* readability */
 #define SCALE_MODE 0
 #define RESCALE_MODE 1
@@ -73,14 +86,15 @@
 #include <idas/idas_spbcgs.h>
 #include <idas/idas_sptfqmr.h>
 
+/* Function prototypes */
 static int callDenseJacobian(long int Neq, double tt, double cj,
-    N_Vector yy, N_Vector yp, N_Vector rr,
-    DlsMat Jac, void *user_data,
-    N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
+                             N_Vector yy, N_Vector yp, N_Vector rr,
+                             DlsMat Jac, void *user_data,
+                             N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
 
-static int callSparseJacobian(realtype tt, realtype cj,
-    N_Vector yy, N_Vector yp, N_Vector rr, SlsMat Jac, void *user_data,
-    N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
+static int callSparseJacobian(realtype currentTime, realtype cj,
+                              N_Vector yy, N_Vector yp, N_Vector rr, SlsMat Jac, void *user_data,
+                              N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
 
 static int residualFunctionIDA(double time, N_Vector yy, N_Vector yp, N_Vector res, void* userData);
 int rootsFunctionIDA(double time, N_Vector yy, N_Vector yp, double *gout, void* userData);
@@ -92,9 +106,12 @@ static int idaReScaleData(IDA_SOLVER *idaData);
 static int idaScaleVector(N_Vector vec, double* factors, unsigned int size);
 static int idaReScaleVector(N_Vector vec, double* factors, unsigned int size);
 
+int ida_event_update(DATA* data, threadData_t *threadData);
+
+/* Static variables */
 static IDA_SOLVER *idaDataGlobal;
 static int initializedSolver = 0;
-int ida_event_update(DATA* data, threadData_t *threadData);
+
 
 int checkIDAflag(int flag)
 {
@@ -115,7 +132,7 @@ int checkIDAflag(int flag)
 }
 
 void errOutputIDA(int error_code, const char *module, const char *function,
-    char *msg, void *userData)
+                  char *msg, void *userData)
 {
   TRACE_PUSH
   DATA* data = (DATA*)(((IDA_USERDATA*)((IDA_SOLVER*)userData)->simData)->data);
@@ -126,10 +143,10 @@ void errOutputIDA(int error_code, const char *module, const char *function,
   TRACE_POP
 }
 
-/* initial main ida data */
-int
-ida_solver_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo, IDA_SOLVER *idaData){
-
+/* initialize main ida data */
+int ida_solver_initial(DATA* data, threadData_t *threadData,
+                       SOLVER_INFO* solverInfo, IDA_SOLVER *idaData)
+{
   TRACE_PUSH
 
   int flag;
@@ -157,6 +174,7 @@ ida_solver_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo
   idaData->residualFunction = residualFunctionIDA;
   idaData->N = (long int)data->modelData->nStates;
 
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
 
   /* change parameter for DAE mode */
   if (compiledInDAEMode)
@@ -262,11 +280,11 @@ ida_solver_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo
     /* daeMode: set nominal values for algebraic variables */
     if (idaData->daeMode)
     {
-     getAlgebraicDAEVarNominals(data, idaData->yScale + data->modelData->nStates);
-     for(i=data->modelData->nStates; i < idaData->N; ++i)
-     {
-       idaData->ypScale[i] = 1.0;
-     }
+      getAlgebraicDAEVarNominals(data, idaData->yScale + data->modelData->nStates);
+      for(i=data->modelData->nStates; i < idaData->N; ++i)
+      {
+        idaData->ypScale[i] = 1.0;
+      }
     }
     infoStreamPrint(LOG_SOLVER_V, 1, "The scale factors for all ida states: ");
     for(i=0; i < idaData->N; ++i)
@@ -414,6 +432,16 @@ ida_solver_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo
       warningStreamPrint(LOG_STDOUT, 1, "Symbolic Jacobians are currently not supported by DAE Mode. Switch back to numerical Jacobian!");
       idaData->jacobianMethod = COLOREDNUMJAC;
     }
+    if(idaData->jacobianMethod == SYMJAC)
+    {
+      warningStreamPrint(LOG_STDOUT, 1, "Symbolic Jacobians without coloring are currently not supported by IDA. Colored symbolical Jacobian will be used.");
+      idaData->jacobianMethod = COLOREDSYMJAC;
+    }
+    if(idaData->jacobianMethod == NUMJAC)
+    {
+      warningStreamPrint(LOG_STDOUT, 1, "Numerical Jacobians without coloring are currently not supported by IDA. Colored numerical Jacobian will be used.");
+      idaData->jacobianMethod = COLOREDNUMJAC;
+    }
     if(idaData->jacobianMethod == JAC_UNKNOWN)
     {
       if (ACTIVE_WARNING_STREAM(LOG_SOLVER))
@@ -455,7 +483,7 @@ ida_solver_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo
       ANALYTIC_JACOBIAN* jac = &data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A];
       infoStreamPrint(LOG_SIMULATION, 1, "Initialized colored Jacobian:");
       infoStreamPrint(LOG_SIMULATION, 0, "columns: %d rows: %d", jac->sizeCols, jac->sizeRows);
-      infoStreamPrint(LOG_SIMULATION, 0, "NNZ:  %d colors: %d", jac->sparsePattern.numberOfNoneZeros, jac->sparsePattern.maxColors);
+      infoStreamPrint(LOG_SIMULATION, 0, "NNZ:  %d colors: %d", jac->sparsePattern->numberOfNoneZeros, jac->sparsePattern->maxColors);
       messageClose(LOG_SIMULATION);
     }
 
@@ -471,7 +499,7 @@ ida_solver_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo
     }
     else
     {
-      idaData->NNZ = data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A].sparsePattern.numberOfNoneZeros;
+      idaData->NNZ = data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A].sparsePattern->numberOfNoneZeros;
       flag = IDAKLU(idaData->ida_mem, idaData->N, idaData->NNZ);
       /* to add a cj identety matrix */
       idaData->tmpJac = NewSparseMat(idaData->N, idaData->N, idaData->N);
@@ -480,12 +508,18 @@ ida_solver_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo
       throwStreamPrint(threadData, "##IDA## Setup of linear solver KLU failed!");
     }
 
+    idaData->allocatedParMem = 0;   /* false */
+
     switch (idaData->jacobianMethod){
     case SYMJAC:
     case NUMJAC:
     case COLOREDSYMJAC:
     case COLOREDNUMJAC:
       flag = IDASlsSetSparseJacFn(idaData->ida_mem, callSparseJacobian);
+#ifdef USE_PARJAC
+      allocateThreadLocalJacobians(data, &(idaData->jacColumns));
+      idaData->allocatedParMem = 1;   /* true */
+#endif
       break;
     default:
       throwStreamPrint(threadData,"For the klu solver jacobian calculation method has to be %s or %s", JACOBIAN_METHOD[COLOREDSYMJAC], JACOBIAN_METHOD[COLOREDNUMJAC]);
@@ -501,6 +535,10 @@ ida_solver_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo
     case NUMJAC:
       /* set jacobian function */
       flag = IDADlsSetDenseJacFn(idaData->ida_mem, callDenseJacobian);
+#ifdef USE_PARJAC
+      allocateThreadLocalJacobians(data, &(idaData->jacColumns));
+      idaData->allocatedParMem = 1;   /* true */
+#endif
       break;
     case INTERNALNUMJAC:
       break;
@@ -595,8 +633,6 @@ ida_solver_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo
     infoStreamPrint(LOG_SOLVER, 0, "initial step size is set automatically.");
   }
 
-
-
   /* configure the sensitivities part */
   idaData->idaSmode = omc_flag[FLAG_IDAS] ? 1 : 0;
 
@@ -655,17 +691,18 @@ ida_solver_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo
   }
   messageClose(LOG_SOLVER);
 
+  if (measure_time_flag) rt_clear(SIM_TIMER_SOLVER); /* Initialization should not add this timer... */
+
   free(tmp);
   TRACE_POP
   return 0;
 }
 
-/* deinitial ida data */
-int
-ida_solver_deinitial(IDA_SOLVER *idaData){
+/* deinitialize ida data */
+int ida_solver_deinitial(IDA_SOLVER *idaData)
+{
   TRACE_PUSH
 
-
   free(idaData->simData);
   free(idaData->ysave);
   free(idaData->ypsave);
@@ -690,15 +727,20 @@ ida_solver_deinitial(IDA_SOLVER *idaData){
   N_VDestroy_Serial(idaData->errwgt);
   N_VDestroy_Serial(idaData->newdelta);
 
+#ifdef USE_PARJAC
+  if (idaData->allocatedParMem) {
+      freeAnalyticalJacobian(&(idaData->jacColumns));
+      idaData->allocatedParMem = 0;
+  }
+#endif
+
   IDAFree(&idaData->ida_mem);
 
   TRACE_POP
   return 0;
 }
 
-
-int
-ida_event_update(DATA* data, threadData_t *threadData)
+int ida_event_update(DATA* data, threadData_t *threadData)
 {
   IDA_SOLVER *idaData = idaDataGlobal;
   int flag;
@@ -714,8 +756,10 @@ ida_event_update(DATA* data, threadData_t *threadData)
       memcpy(idaData->statesDer, data->localData[0]->realVars + data->modelData->nStates, sizeof(double)*data->modelData->nStates);
 
       /* update inner algebraic get new values from data */
+      if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
       evaluateDAEResiduals_wrapperEventUpdate(data, threadData);
       getAlgebraicDAEVars(data, idaData->states + data->modelData->nStates);
+      if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
 
       infoStreamPrint(LOG_SOLVER, 0, "##IDA## do event update at %.15g", data->localData[0]->timeValue);
       flag = IDAReInit(idaData->ida_mem,
@@ -758,25 +802,30 @@ ida_event_update(DATA* data, threadData_t *threadData)
       IDAGetConsistentIC(idaData->ida_mem, idaData->y, idaData->yp);
 
       /* update inner algebraic variables */
+      if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
       evaluateDAEResiduals_wrapperEventUpdate(data, threadData);
 
       memcpy(data->localData[0]->realVars, idaData->states, sizeof(double)*data->modelData->nStates);
       // and  also algebraic vars
       setAlgebraicDAEVars(data, idaData->states + data->modelData->nStates);
       memcpy(data->localData[0]->realVars + data->modelData->nStates, idaData->statesDer, sizeof(double)*data->modelData->nStates);
+      if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
 
       /* reset initial step size again to default */
       IDASetInitStep(idaData->ida_mem, 0.0);
     }
   }
   else{
+    if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
     data->callback->functionDAE(data, threadData);
+    if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
   }
+
+  return 0;
 }
 
 /* main ida function to make a step */
-int
-ida_solver_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
+int ida_solver_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
 {
   TRACE_PUSH
   double tout = 0;
@@ -788,6 +837,8 @@ ida_solver_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
   int stepsMode;
   int restartAfterLSFail = 0;
 
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
+
   IDA_SOLVER *idaData = (IDA_SOLVER*) solverInfo->solverData;
 
   SIMULATION_DATA *sData = data->localData[0];
@@ -890,6 +941,7 @@ ida_solver_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
       NV_Ith_S(idaData->y, i) = NV_Ith_S(idaData->y, i) + NV_Ith_S(idaData->yp, i) * solverInfo->currentStepSize;
     }
     sData->timeValue = solverInfo->currentTime + solverInfo->currentStepSize;
+    if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
     data->callback->functionODE(data, threadData);
     solverInfo->currentTime = sData->timeValue;
 
@@ -927,9 +979,11 @@ ida_solver_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
   {
     infoStreamPrint(LOG_SOLVER, 1, "##IDA## new step from %.15g to %.15g", solverInfo->currentTime, tout);
 
+    if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
     /* read input vars */
     externalInputUpdate(data);
     data->callback->input_function(data, threadData);
+    if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
 
     if (omc_flag[FLAG_IDA_SCALING])
     {
@@ -1118,6 +1172,7 @@ ida_solver_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
   }
 
   infoStreamPrint(LOG_SOLVER, 0, "##IDA## Finished Integrator step.");
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
 
   TRACE_POP
   return retVal;
@@ -1186,28 +1241,34 @@ int residualFunctionIDA(double time, N_Vector yy, N_Vector yp, N_Vector res, voi
   }
 
   /* read input vars */
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
   externalInputUpdate(data);
   data->callback->input_function(data, threadData);
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
 
   if (idaData->daeMode)
   {
     /* eval residual vars */
+    if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
     data->simulationInfo->daeModeData->evaluateDAEResiduals(data, threadData, EVAL_DYNAMIC);
+    if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
     /* get residual variables */
     for(i=0; i < idaData->N; i++)
     {
       NV_Ith_S(res, i) = data->simulationInfo->daeModeData->residualVars[i];
-      infoStreamPrint(LOG_SOLVER_V, 0, "%d. residual = %e", i, NV_Ith_S(res, i));
+      infoStreamPrint(LOG_SOLVER_V, 0, "%ld. residual = %e", i, NV_Ith_S(res, i));
     }
   }
   else
   {
     /* eval function ODE */
+    if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
     data->callback->functionODE(data, threadData);
+    if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
     for(i=0; i < idaData->N; i++)
     {
       NV_Ith_S(res, i) = data->localData[0]->realVars[data->modelData->nStates + i] - NV_Ith_S(yp, i);
-      infoStreamPrint(LOG_SOLVER_V, 0, "%d. residual = %e", i, NV_Ith_S(res, i));
+      infoStreamPrint(LOG_SOLVER_V, 0, "%ld. residual = %e", i, NV_Ith_S(res, i));
     }
   }
 
@@ -1236,6 +1297,7 @@ int residualFunctionIDA(double time, N_Vector yy, N_Vector yp, N_Vector res, voi
     unsetContext(data);
   }
   messageClose(LOG_SOLVER_V);
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
 
   TRACE_POP
   return retVal;
@@ -1278,6 +1340,7 @@ int rootsFunctionIDA(double time, N_Vector yy, N_Vector yp, double *gout, void*
   data->localData[0]->timeValue = time;
 
   /* read input vars */
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
   externalInputUpdate(data);
   data->callback->input_function(data, threadData);
   /* eval needed equations*/
@@ -1290,6 +1353,7 @@ int rootsFunctionIDA(double time, N_Vector yy, N_Vector yp, double *gout, void*
   }
 
   data->callback->function_ZeroCrossings(data, threadData, gout);
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
 
   threadData->currentErrorStage = saveJumpState;
 
@@ -1303,19 +1367,20 @@ int rootsFunctionIDA(double time, N_Vector yy, N_Vector yp, double *gout, void*
     unsetContext(data);
   }
   messageClose(LOG_SOLVER_V);
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
 
   TRACE_POP
   return 0;
 }
 
-
 /*
  *  function calculates a jacobian matrix by
  *  numerical method finite differences with coloring
  *  into a dense DlsMat matrix
  */
 static
-int jacColoredNumericalDense(double tt, N_Vector yy, N_Vector yp, N_Vector rr, DlsMat Jac, double cj, void *userData)
+int jacColoredNumericalDense(double currentTime, N_Vector yy, N_Vector yp,
+                             N_Vector rr, DlsMat Jac, double cj, void *userData)
 {
   TRACE_PUSH
   IDA_SOLVER* idaData = (IDA_SOLVER*)userData;
@@ -1355,10 +1420,10 @@ int jacColoredNumericalDense(double tt, N_Vector yy, N_Vector yp, N_Vector rr, D
   }
   else
   {
-    sparsePattern = &(data->simulationInfo->analyticJacobians[index].sparsePattern);
+    sparsePattern = data->simulationInfo->analyticJacobians[index].sparsePattern;
   }
 
-  setContext(data, &tt, CONTEXT_JACOBIAN);
+  setContext(data, &currentTime, CONTEXT_JACOBIAN);
 
   for(i = 0; i < sparsePattern->maxColors; i++)
   {
@@ -1382,7 +1447,7 @@ int jacColoredNumericalDense(double tt, N_Vector yy, N_Vector yp, N_Vector rr, D
       }
     }
 
-    (*idaData->residualFunction)(tt, yy, yp, idaData->newdelta, userData);
+    (*idaData->residualFunction)(currentTime, yy, yp, idaData->newdelta, userData);
 
     increaseJacContext(data);
 
@@ -1412,43 +1477,65 @@ int jacColoredNumericalDense(double tt, N_Vector yy, N_Vector yp, N_Vector rr, D
 }
 
 /*
- *  function calculates the Jacobian matrix symbolical
- *  with considering also the coloring and pass it in a
- *  dense DlsMat matrix.
+ *  function calculates the Jacobian matrix symbolically with considering also
+ *  the coloring and pass it in a dense DlsMat matrix.
  */
-static
-int jacColoredSymbolicalDense(double tt, N_Vector yy, N_Vector yp, N_Vector rr, DlsMat Jac, double cj, void *userData)
+static int jacColoredSymbolicalDense(double currentTime, N_Vector yy,
+                                     N_Vector yp, N_Vector rr, DlsMat Jac,
+                                     double cj, void *userData)
 {
   TRACE_PUSH
   IDA_SOLVER* idaData = (IDA_SOLVER*)userData;
   DATA* data = (DATA*)(((IDA_USERDATA*)idaData->simData)->data);
   threadData_t* threadData = (threadData_t*)(((IDA_USERDATA*)idaData->simData)->threadData);
   void* ida_mem = idaData->ida_mem;
+  long int N = idaData->N;
   const int index = data->callback->INDEX_JAC_A;
   unsigned int i,ii,j, nth;
-  ANALYTIC_JACOBIAN* jacData = &(data->simulationInfo->analyticJacobians[index]);
-  SPARSE_PATTERN* sparsePattern = &(jacData->sparsePattern);
+  SPARSE_PATTERN* sparsePattern = data->simulationInfo->analyticJacobians[index].sparsePattern;
 
   /* prepare variables */
   double *states = N_VGetArrayPointer(yy);
   double *yprime = N_VGetArrayPointer(yp);
 
-  setContext(data, &tt, CONTEXT_SYM_JACOBIAN);
+  setContext(data, &currentTime, CONTEXT_SYM_JACOBIAN);      /* Reuse jacobian matrix in KLU solver */
 
+#ifdef USE_PARJAC
+  GC_allow_register_threads();
+#endif
+
+#pragma omp parallel default(none) firstprivate(N) shared(i, sparsePattern, idaData, data, threadData, Jac) private(ii, j, nth)
+{
+#ifdef USE_PARJAC
+  /* Register omp-thread in GC */
+  if(!GC_thread_is_registered()) {
+     struct GC_stack_base sb;
+     memset (&sb, 0, sizeof(sb));
+     GC_get_stack_base(&sb);
+     GC_register_my_thread (&sb);
+  }
+  // ToDo Use always a thread local analyticJacobians (replace simulationInfo->analyticaJacobians)
+  // These are not the Jacobians of the linear systems! (SimulationInfo->linearSystemData[idx].jacobian)
+  ANALYTIC_JACOBIAN* t_jac = &(idaData->jacColumns[omc_get_thread_num()]);
+#else
+  ANALYTIC_JACOBIAN* t_jac = &(data->simulationInfo->analyticJacobians[index]);
+#endif
+
+#pragma omp for
   for(i = 0; i < sparsePattern->maxColors; i++)
   {
-    for(ii=0; ii < idaData->N; ii++)
+    for(ii=0; ii < N; ii++)
     {
       if(sparsePattern->colorCols[ii]-1 == i)
       {
-        jacData->seedVars[ii] = 1;
+        t_jac->seedVars[ii] = 1;
       }
     }
 
-    data->callback->functionJacA_column(data, threadData, jacData, NULL);
+    data->callback->functionJacA_column(data, threadData, t_jac, NULL);
     increaseJacContext(data);
 
-    for(ii = 0; ii < idaData->N; ii++)
+    for(ii = 0; ii < N; ii++)
     {
       if(sparsePattern->colorCols[ii]-1 == i)
       {
@@ -1456,8 +1543,8 @@ int jacColoredSymbolicalDense(double tt, N_Vector yy, N_Vector yp, N_Vector rr,
         while(nth < sparsePattern->leadindex[ii+1])
         {
           j  =  sparsePattern->index[nth];
-          infoStreamPrint(LOG_JAC, 0, "### symbolical jacobian  at [%d,%d] = %f ###", j, ii, jacData->resultVars[j]);
-          DENSE_ELEM(Jac, j, ii) = jacData->resultVars[j];
+          infoStreamPrint(LOG_JAC, 0, "### symbolical jacobian  at [%d,%d] = %f ###", j, ii, t_jac->resultVars[j]);
+          DENSE_ELEM(Jac, j, ii) = t_jac->resultVars[j];
           nth++;
         };
       }
@@ -1465,9 +1552,11 @@ int jacColoredSymbolicalDense(double tt, N_Vector yy, N_Vector yp, N_Vector rr,
 
     for(ii=0; ii < idaData->N; ii++)
     {
-      jacData->seedVars[ii] = 0;
+      t_jac->seedVars[ii] = 0;
     }
-  }
+  } // for column
+} // omp parallel
+
   unsetContext(data);
 
   TRACE_POP
@@ -1488,6 +1577,7 @@ static int callDenseJacobian(long int Neq, double tt, double cj,
   int retVal;
 
   /* profiling */
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
   rt_tick(SIM_TIMER_JACOBIAN);
 
   if (idaData->jacobianMethod == COLOREDNUMJAC || idaData->jacobianMethod == NUMJAC)
@@ -1503,9 +1593,6 @@ static int callDenseJacobian(long int Neq, double tt, double cj,
     throwStreamPrint(threadData, "##IDA## Something goes wrong while obtain jacobian matrix!");
   }
 
-  /* profiling */
-  rt_accumulate(SIM_TIMER_JACOBIAN);
-
   /* debug */
   if (ACTIVE_STREAM(LOG_JAC)){
     _omc_matrix* dumpJac = _omc_createMatrix(idaData->N, idaData->N, Jac->data);
@@ -1523,18 +1610,24 @@ static int callDenseJacobian(long int Neq, double tt, double cj,
     }
   }
 
+  /* profiling */
+  rt_accumulate(SIM_TIMER_JACOBIAN);
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
+
   TRACE_POP
   return retVal;
 }
 
 /* Element function for sparse matrix set */
-static void setJacElementKluSparse(int row, int col, double value, int nth, SlsMat spJac)
+static void setJacElementKluSparse(int row, int col, int nth, double value, void* spJac, int rows)
 {
-  if (col > 0 && spJac->colptrs[col] == 0){
-      spJac->colptrs[col] = nth;
+  (void) rows; // Unused, needed to match genericColoredSymbolicJacobianEvaluation
+  SlsMat mat = (SlsMat)spJac;
+  if (col > 0 && mat->colptrs[col] == 0){
+      mat->colptrs[col] = nth;
   }
-  spJac->rowvals[nth] = row;
-  spJac->data[nth] = value;
+  mat->rowvals[nth] = row;
+  mat->data[nth] = value;
 }
 
 /* finish sparse matrix, by fixing colprts */
@@ -1555,8 +1648,7 @@ static void finishSparseColPtr(SlsMat mat, int nnz)
  *  numerical method finite differences with coloring
  *  into a sparse SlsMat matrix
  */
-static
-int jacoColoredNumericalSparse(double tt, N_Vector yy, N_Vector yp, N_Vector rr, SlsMat Jac, double cj, void *userData)
+static int jacoColoredNumericalSparse(double currentTime, N_Vector yy, N_Vector yp, N_Vector rr, SlsMat Jac, double cj, void *userData)
 {
   TRACE_PUSH
   IDA_SOLVER* idaData = (IDA_SOLVER*)userData;
@@ -1601,13 +1693,13 @@ int jacoColoredNumericalSparse(double tt, N_Vector yy, N_Vector yp, N_Vector rr,
   }
   else
   {
-    sparsePattern = &(data->simulationInfo->analyticJacobians[index].sparsePattern);
+    sparsePattern = data->simulationInfo->analyticJacobians[index].sparsePattern;
   }
 
   /* it's needed to clear the matrix */
   SlsSetToZero(Jac);
 
-  setContext(data, &tt, CONTEXT_JACOBIAN);
+  setContext(data, &currentTime, CONTEXT_JACOBIAN);
 
   /* rescale idaData->y and idaData->yp
    * the evaluation of the  residual function
@@ -1641,7 +1733,7 @@ int jacoColoredNumericalSparse(double tt, N_Vector yy, N_Vector yp, N_Vector rr,
       }
     }
     idaData->disableScaling = 1;
-    (*idaData->residualFunction)(tt, yy, yp, idaData->newdelta, userData);
+    (*idaData->residualFunction)(currentTime, yy, yp, idaData->newdelta, userData);
     idaData->disableScaling = disBackup;
 
     increaseJacContext(data);
@@ -1654,12 +1746,12 @@ int jacoColoredNumericalSparse(double tt, N_Vector yy, N_Vector yp, N_Vector rr,
         while(nth < sparsePattern->leadindex[ii+1])
         {
           j  =  sparsePattern->index[nth];
-          //setJacElementKluSparse(j, ii, (newdelta[j] - delta[j]) * delta_hh[ii], nth, Jac);
+          //setJacElementKluSparse(j, ii, nth, (newdelta[j] - delta[j]) * delta_hh[ii], Jac, -1);
           /* use row scaling for jacobian elements */
           if (idaData->disableScaling == 1 || !omc_flag[FLAG_IDA_SCALING]){
-            setJacElementKluSparse(j, ii, (newdelta[j] - delta[j]) * delta_hh[ii], nth, Jac);
+            setJacElementKluSparse(j, ii, nth, (newdelta[j] - delta[j]) * delta_hh[ii], Jac, -1);
           }else{
-            setJacElementKluSparse(j, ii, ((newdelta[j] - delta[j]) * delta_hh[ii]) / idaData->resScale[j] * idaData->yScale[ii], nth, Jac);
+            setJacElementKluSparse(j, ii, nth, ((newdelta[j] - delta[j]) * delta_hh[ii]) / idaData->resScale[j] * idaData->yScale[ii], Jac, -1);
           }
           nth++;
         };
@@ -1687,66 +1779,41 @@ int jacoColoredNumericalSparse(double tt, N_Vector yy, N_Vector yp, N_Vector rr,
   return 0;
 }
 
-/* \fn jacColoredSymbolicalSparse(double tt, N_Vector yy, N_Vector yp, N_Vector rr, SlsMat Jac, double cj, void *userData)
- *
- *
- * This function calculates symbolically the jacobian matrix and exploiting the coloring.
+/*
+ * This function calculates the jacobian matrix symbolically while exploiting coloring.
  */
-int
-jacColoredSymbolicalSparse(double tt, N_Vector yy, N_Vector yp, N_Vector rr, SlsMat Jac, double cj, void *userData)
+int jacColoredSymbolicalSparse(double currentTime, N_Vector yy, N_Vector yp,
+                               N_Vector rr, SlsMat Jac, double cj,
+                               void *userData)
 {
   TRACE_PUSH
   IDA_SOLVER* idaData = (IDA_SOLVER*)userData;
   DATA* data = (DATA*)(((IDA_USERDATA*)idaData->simData)->data);
   threadData_t* threadData = (threadData_t*)(((IDA_USERDATA*)idaData->simData)->threadData);
-  void* ida_mem = idaData->ida_mem;
   const int index = data->callback->INDEX_JAC_A;
-  unsigned int i,ii,j,nth;
 
   /* prepare variables */
   double *states = N_VGetArrayPointer(yy);
   double *yprime = N_VGetArrayPointer(yp);
 
+#ifdef USE_PARJAC
+  ANALYTIC_JACOBIAN* jacData = (idaData->jacColumns);
+#else
   ANALYTIC_JACOBIAN* jacData = &(data->simulationInfo->analyticJacobians[index]);
-  SPARSE_PATTERN* sparsePattern = &(jacData->sparsePattern);
+#endif
+  unsigned int columns = jacData->sizeCols;
+  unsigned int rows = jacData->sizeRows;
+  SPARSE_PATTERN* sparsePattern = jacData->sparsePattern;
+  int maxColors = sparsePattern->maxColors;
 
   /* it's needed to clear the matrix */
   SlsSetToZero(Jac);
 
-  setContext(data, &tt, CONTEXT_SYM_JACOBIAN);
+  setContext(data, &currentTime, CONTEXT_SYM_JACOBIAN);      /* Reuse jacobian matrix in KLU solver */
 
-  for(i = 0; i < sparsePattern->maxColors; i++)
-  {
-    for(ii=0; ii < idaData->N; ii++)
-    {
-      if(sparsePattern->colorCols[ii]-1 == i)
-      {
-        jacData->seedVars[ii] = 1;
-      }
-    }
+  genericColoredSymbolicJacobianEvaluation(rows, columns, sparsePattern, Jac, jacData,
+                                           data, threadData, &setJacElementKluSparse);
 
-    data->callback->functionJacA_column(data, threadData, jacData, NULL);
-    increaseJacContext(data);
-
-    for(ii = 0; ii < idaData->N; ii++)
-    {
-      if(sparsePattern->colorCols[ii]-1 == i)
-      {
-        nth = sparsePattern->leadindex[ii];
-        while(nth < sparsePattern->leadindex[ii+1])
-        {
-          j  =  sparsePattern->index[nth];
-          setJacElementKluSparse(j, ii, jacData->resultVars[j], nth, Jac);
-          nth++;
-        };
-      }
-    }
-
-    for(ii=0; ii < idaData->N; ii++)
-    {
-      jacData->seedVars[ii] = 0;
-    }
-  }
   finishSparseColPtr(Jac, sparsePattern->numberOfNoneZeros);
   unsetContext(data);
 
@@ -1757,33 +1824,30 @@ jacColoredSymbolicalSparse(double tt, N_Vector yy, N_Vector yp, N_Vector rr, Sls
 /*
  * Wrapper function to call numerical or symbolical jacobian matrix
  */
-static int callSparseJacobian(double tt, double cj,
-    N_Vector yy, N_Vector yp, N_Vector rr,
-    SlsMat Jac, void *user_data,
-    N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
+static int callSparseJacobian(double currentTime, double cj,
+                              N_Vector yy, N_Vector yp, N_Vector rr,
+                              SlsMat Jac, void *user_data,
+                              N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
 {
   TRACE_PUSH
-  int retVal;
 
   IDA_SOLVER* idaData = (IDA_SOLVER*)user_data;
   DATA* data = (DATA*)(((IDA_USERDATA*)idaData->simData)->data);
   threadData_t* threadData = (threadData_t*)(((IDA_USERDATA*)((IDA_SOLVER*)user_data)->simData)->threadData);
 
   /* profiling */
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
   rt_tick(SIM_TIMER_JACOBIAN);
 
   if (idaData->jacobianMethod == COLOREDSYMJAC || idaData->jacobianMethod == SYMJAC)
   {
-    retVal = jacColoredSymbolicalSparse(tt, yy, yp, rr, Jac, cj, user_data);
+    jacColoredSymbolicalSparse(currentTime, yy, yp, rr, Jac, cj, user_data);
   }
   else if (idaData->jacobianMethod == COLOREDNUMJAC || idaData->jacobianMethod == NUMJAC)
   {
-    retVal = jacoColoredNumericalSparse(tt, yy, yp, rr, Jac, cj, user_data);
+    jacoColoredNumericalSparse(currentTime, yy, yp, rr, Jac, cj, user_data);
   }
 
-  /* profiling */
-  rt_accumulate(SIM_TIMER_JACOBIAN);
-
   /* debug */
   if (ACTIVE_STREAM(LOG_JAC)){
     infoStreamPrint(LOG_JAC, 0, "##IDA## Sparse Matrix A.");
@@ -1803,12 +1867,15 @@ static int callSparseJacobian(double tt, double cj,
     SlsAddMat(Jac, idaData->tmpJac);
   }
 
+  /* profiling */
+  rt_accumulate(SIM_TIMER_JACOBIAN);
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
+
   TRACE_POP
-  return retVal;
+  return 0;
 }
 
-static
-int getScalingFactors(DATA* data, IDA_SOLVER *idaData, SlsMat inScaleMatrix)
+static int getScalingFactors(DATA* data, IDA_SOLVER *idaData, SlsMat inScaleMatrix)
 {
   int i;
 
@@ -1878,8 +1945,7 @@ int getScalingFactors(DATA* data, IDA_SOLVER *idaData, SlsMat inScaleMatrix)
   return 0;
 }
 
-static
-int idaScaleVector(N_Vector vec, double* factors, unsigned int size)
+static int idaScaleVector(N_Vector vec, double* factors, unsigned int size)
 {
   int i;
   double *data = N_VGetArrayPointer(vec);
@@ -1891,8 +1957,8 @@ int idaScaleVector(N_Vector vec, double* factors, unsigned int size)
   printVector(LOG_SOLVER_V, "scaled", data, size, 0.0);
   return 0;
 }
-static
-int idaReScaleVector(N_Vector vec, double* factors, unsigned int size)
+
+static int idaReScaleVector(N_Vector vec, double* factors, unsigned int size)
 {
   int i;
   double *data = N_VGetArrayPointer(vec);
@@ -1906,8 +1972,7 @@ int idaReScaleVector(N_Vector vec, double* factors, unsigned int size)
   return 0;
 }
 
-static
-int idaScaleData(IDA_SOLVER *idaData)
+static int idaScaleData(IDA_SOLVER *idaData)
 {
   infoStreamPrint(LOG_SOLVER_V, 1, "Scale y");
   idaScaleVector(idaData->y, idaData->yScale, idaData->N);
@@ -1919,8 +1984,7 @@ int idaScaleData(IDA_SOLVER *idaData)
   return 0;
 }
 
-static
-int idaReScaleData(IDA_SOLVER *idaData)
+static int idaReScaleData(IDA_SOLVER *idaData)
 {
   infoStreamPrint(LOG_SOLVER_V, 1, "Re-Scale y");
   idaReScaleVector(idaData->y, idaData->yScale, idaData->N);
@@ -1933,5 +1997,4 @@ int idaReScaleData(IDA_SOLVER *idaData)
 }
 
 
-
 #endif
