@@ -39,12 +39,12 @@ extern "C" {
 #include <stdlib.h>
 #include <string.h> /* memcpy */
 
-#include "simulation/simulation_info_json.h"
-#include "util/omc_error.h"
-#include "util/varinfo.h"
+#include "../simulation_info_json.h"
+#include "../../util/omc_error.h"
+#include "../../util/varinfo.h"
 #include "model_help.h"
-#include "gc/omc_gc.h"
-#include "meta/meta_modelica.h"
+#include "../../gc/omc_gc.h"
+#include "../../meta/meta_modelica.h"
 
 #include "nonlinearSystem.h"
 #include "nonlinearSolverHybrd.h"
@@ -56,7 +56,7 @@ struct dataAndSys {
   int sysNumber;
 };
 
-static int wrapper_fvec_hybrj(const integer* n, const double* x, double* f, double* fjac, const integer* ldjac, const integer* iflag, void* data);
+static void wrapper_fvec_hybrj(const integer* n, const double* x, double* f, double* fjac, const integer* ldjac, const integer* iflag, void* data);
 
 /*! \fn allocate memory for nonlinear system solver hybrd
  *
@@ -268,11 +268,15 @@ static int getAnalyticalJacobian(struct dataAndSys* dataSys, double* jac)
   memset(jac, 0, (solverData->n)*(solverData->n)*sizeof(double));
   memset(solverData->fjacobian, 0, (solverData->n)*(solverData->n)*sizeof(double));
 
-  for(i=0; i < jacobian->sparsePattern.maxColors; i++)
+  if (jacobian->constantEqns != NULL) {
+    jacobian->constantEqns(data, threadData, jacobian, NULL);
+  }
+
+  for(i=0; i < jacobian->sparsePattern->maxColors; i++)
   {
     /* activate seed variable for the corresponding color */
     for(ii=0; ii < jacobian->sizeCols; ii++)
-      if(jacobian->sparsePattern.colorCols[ii]-1 == i)
+      if(jacobian->sparsePattern->colorCols[ii]-1 == i)
         jacobian->seedVars[ii] = 1;
 
     ((systemData->analyticalJacobianColumn))(data, threadData, jacobian, NULL);
@@ -281,17 +285,17 @@ static int getAnalyticalJacobian(struct dataAndSys* dataSys, double* jac)
     {
       if(jacobian->seedVars[j] == 1)
       {
-        ii = jacobian->sparsePattern.leadindex[j];
-        while(ii < jacobian->sparsePattern.leadindex[j+1])
+        ii = jacobian->sparsePattern->leadindex[j];
+        while(ii < jacobian->sparsePattern->leadindex[j+1])
         {
-          l  = jacobian->sparsePattern.index[ii];
+          l  = jacobian->sparsePattern->index[ii];
           k  = j*jacobian->sizeRows + l;
           solverData->fjacobian[k] = jac[k] = jacobian->resultVars[l];
           ii++;
         };
       }
       /* de-activate seed variable for the corresponding color */
-      if(jacobian->sparsePattern.colorCols[j]-1 == i)
+      if(jacobian->sparsePattern->colorCols[j]-1 == i)
         jacobian->seedVars[j] = 0;
     }
 
@@ -305,7 +309,7 @@ static int getAnalyticalJacobian(struct dataAndSys* dataSys, double* jac)
  *
  *
  */
-static int wrapper_fvec_hybrj(const integer* n, const double* x, double* f, double* fjac, const integer* ldjac, const integer* iflag, void* dataAndSysNum)
+static void wrapper_fvec_hybrj(const integer* n, const double* x, double* f, double* fjac, const integer* ldjac, const integer* iflag, void* dataAndSysNum)
 {
   int i,j;
   struct dataAndSys *dataSys = (struct dataAndSys*) dataAndSysNum;
@@ -400,8 +404,6 @@ static int wrapper_fvec_hybrj(const integer* n, const double* x, double* f, doub
     throwStreamPrint(NULL, "Well, this is embarrasing. The non-linear solver should never call this case.%d", (int)*iflag);
     break;
   }
-
-  return 0;
 }
 
 /*! \fn solve non-linear system with hybrd method
