@@ -31,10 +31,11 @@
 
 #include "real_array.h"
 #include "index_spec.h"
-#include "memory_pool.h"
+#include "../gc/omc_gc.h"
 #include "division.h"
 #include "integer_array.h"
 #include "omc_error.h"
+#include "generic_array.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -92,17 +93,7 @@ void alloc_real_array_data(real_array_t *a)
 
 void copy_real_array_data(const real_array_t source, real_array_t *dest)
 {
-    size_t i, nr_of_elements;
-
-    omc_assert_macro(base_array_ok(&source));
-    omc_assert_macro(base_array_ok(dest));
-    omc_assert_macro(base_array_shape_eq(&source, dest));
-
-    nr_of_elements = base_array_nr_of_elements(source);
-
-    for(i = 0; i < nr_of_elements; ++i) {
-        real_set(dest, i, real_get(source, i));
-    }
+    real_array_copy_data(source,*dest);
 }
 
 void copy_real_array_data_mem(const real_array_t source, modelica_real *dest)
@@ -120,9 +111,7 @@ void copy_real_array_data_mem(const real_array_t source, modelica_real *dest)
 
 void copy_real_array(const real_array_t source, real_array_t *dest)
 {
-    clone_base_array_spec(&source, dest);
-    alloc_real_array_data(dest);
-    copy_real_array_data(source,dest);
+    real_array_alloc_copy(source,*dest);
 }
 
 
@@ -279,33 +268,9 @@ void simple_indexed_assign_real_array2(const real_array_t * source,
 void indexed_assign_real_array(const real_array_t source, real_array_t* dest,
                                const index_spec_t* dest_spec)
 {
-    _index_t* idx_vec1;
-    _index_t* idx_size;
-    int i,j;
-
-    omc_assert_macro(base_array_ok(&source));
-    omc_assert_macro(base_array_ok(dest));
-    omc_assert_macro(index_spec_ok(dest_spec));
-    omc_assert_macro(index_spec_fit_base_array(dest_spec, dest));
-    for(i = 0,j = 0; i < dest_spec->ndims; ++i) {
-        if(dest_spec->dim_size[i] != 0) {
-            ++j;
-        }
-    }
-    omc_assert_macro(j == source.ndims);
-
-    idx_vec1 = size_alloc(dest->ndims);
-    idx_size = size_alloc(dest_spec->ndims);
-
-    for(i = 0; i < dest_spec->ndims; ++i) {
-        idx_vec1[i] = 0;
-
-        if(dest_spec->index[i] != NULL) { /* is 'S' or 'A' */
-            idx_size[i] = imax(dest_spec->dim_size[i],1);
-        } else { /* is 'W' */
-            idx_size[i] = dest->dim_size[i];
-        }
-    }
+    _index_t *idx_vec1, *idx_size;
+    int j;
+    indexed_assign_base_array_size_alloc(&source, dest, dest_spec, &idx_vec1, &idx_size);
 
     j = 0;
     do {
@@ -413,33 +378,7 @@ void index_alloc_real_array(const real_array_t * source,
                             const index_spec_t* source_spec,
                             real_array_t* dest)
 {
-    int i;
-    int j;
-
-    omc_assert_macro(base_array_ok(source));
-    omc_assert_macro(index_spec_ok(source_spec));
-    omc_assert_macro(index_spec_fit_base_array(source_spec, source));
-
-    for(i = 0, j = 0; i < source_spec->ndims; ++i) {
-         if(source_spec->dim_size[i] != 0) { /* is 'W' or 'A' */
-           ++j;
-         }
-    }
-    dest->ndims = j;
-    dest->dim_size = size_alloc(dest->ndims);
-
-    for(i = 0, j = 0; i < source_spec->ndims; ++i) {
-        if(source_spec->dim_size[i] != 0) { /* is 'W' or 'A' */
-            if(source_spec->index[i] != NULL) { /* is 'A' */
-                dest->dim_size[j] = source_spec->dim_size[i];
-            } else { /* is 'W' */
-                dest->dim_size[j] = source->dim_size[i];
-            }
-
-            ++j;
-        }
-    }
-
+    index_alloc_base_array_size(source, source_spec, dest);
     alloc_real_array_data(dest);
     index_real_array(source, source_spec, dest);
 }
@@ -558,21 +497,6 @@ void array_alloc_real_array(real_array_t* dest, int n, real_array_t first,...)
     free(elts);
 }
 
-void array_scalar_real_array(real_array_t* dest, int n, modelica_real first, ...)
-{
-    int i;
-    va_list ap;
-    omc_assert_macro(base_array_ok(dest));
-    omc_assert_macro(dest->ndims == 1);
-    omc_assert_macro(dest->dim_size[0] == n);
-    put_real_element(first, 0, dest);
-    va_start(ap,first);
-    for(i = 0; i < n; ++i) {
-        put_real_element(va_arg(ap,modelica_real),i,dest);
-    }
-    va_end(ap);
-}
-
 /* array_alloc_scalar_real_array
  *
  * Creates(incl allocation) an array from scalar elements.
@@ -620,13 +544,13 @@ modelica_real* real_array_element_addr(const real_array_t * source,int ndims,...
  * k is one based
  */
 void cat_real_array(int k, real_array_t* dest, int n,
-                    real_array_t* first,...)
+                    const real_array_t* first,...)
 {
     va_list ap;
     int i, j, r, c;
     int n_sub = 1, n_super = 1;
     int new_k_dim_size = 0;
-    real_array_t **elts = (real_array_t**)malloc(sizeof(real_array_t *) * n);
+    const real_array_t **elts = (const real_array_t**)malloc(sizeof(real_array_t *) * n);
 
     omc_assert_macro(elts);
     /* collect all array ptrs to simplify traversal.*/
@@ -634,7 +558,7 @@ void cat_real_array(int k, real_array_t* dest, int n,
     elts[0] = first;
 
     for(i = 1; i < n; i++) {
-        elts[i] = va_arg(ap,real_array_t*);
+        elts[i] = va_arg(ap,const real_array_t*);
     }
     va_end(ap);
 
@@ -682,13 +606,13 @@ void cat_real_array(int k, real_array_t* dest, int n,
  * k is one based
  */
 void cat_alloc_real_array(int k, real_array_t* dest, int n,
-                          real_array_t* first,...)
+                          const real_array_t* first,...)
 {
     va_list ap;
     int i, j, r, c;
     int n_sub = 1, n_super = 1;
     int new_k_dim_size = 0;
-    real_array_t **elts = (real_array_t**)malloc(sizeof(real_array_t *) * n);
+    const real_array_t **elts = (const real_array_t**)malloc(sizeof(real_array_t *) * n);
 
     omc_assert_macro(elts);
     /* collect all array ptrs to simplify traversal.*/
@@ -696,7 +620,7 @@ void cat_alloc_real_array(int k, real_array_t* dest, int n,
     elts[0] = first;
 
     for(i = 1; i < n; i++) {
-        elts[i] = va_arg(ap,real_array_t*);
+        elts[i] = va_arg(ap,const real_array_t*);
     }
     va_end(ap);
 
@@ -785,6 +709,32 @@ real_array_t add_alloc_real_array(const real_array_t a, const real_array_t b)
     return dest;
 }
 
+real_array_t add_alloc_scalar_real_array(modelica_real sc, const real_array_t *arr)
+{
+  size_t nr_of_elements, i;
+  real_array_t dest;
+  clone_real_array_spec(arr, &dest);
+  alloc_real_array_data(&dest);
+  nr_of_elements = base_array_nr_of_elements(*arr);
+  for(i=0; i < nr_of_elements; ++i) {
+    real_set(&dest, i, sc + real_get(*arr, i));
+  }
+  return dest;
+}
+
+real_array_t sub_alloc_scalar_real_array(modelica_real sc, const real_array_t *arr)
+{
+  size_t nr_of_elements, i;
+  real_array_t dest;
+  clone_real_array_spec(arr, &dest);
+  alloc_real_array_data(&dest);
+  nr_of_elements = base_array_nr_of_elements(*arr);
+  for(i=0; i < nr_of_elements; ++i) {
+    real_set(&dest, i, sc - real_get(*arr, i));
+  }
+  return dest;
+}
+
 void usub_real_array(real_array_t* a)
 {
     size_t nr_of_elements, i;
@@ -1407,6 +1357,24 @@ void identity_real_array(int n, real_array_t* dest)
     }
 }
 
+static void diagonal_real_array_impl(const real_array_t *v, real_array_t* dest)
+{
+    size_t i;
+    size_t j;
+    size_t n;
+
+    n = v->dim_size[0];
+
+    for(i = 0; i < (n * n); ++i) {
+        real_set(dest, i, 0);
+    }
+    j = 0;
+    for(i = 0; i < n; ++i) {
+        real_set(dest, j, real_get(*v, i));
+        j += n + 1;
+    }
+}
+
 void diagonal_real_array(const real_array_t * v,real_array_t* dest)
 {
     size_t i;
@@ -1421,35 +1389,20 @@ void diagonal_real_array(const real_array_t * v,real_array_t* dest)
     omc_assert_macro(dest->ndims == 2);
     omc_assert_macro((dest->dim_size[0] == n) && (dest->dim_size[1] == n));
 
-    for(i = 0; i < (n * n); ++i) {
-        real_set(dest, i, 0);
-    }
-    j = 0;
-    for(i = 0; i < n; ++i) {
-        real_set(dest, j, real_get(*v, i));
-        j += n+1;
-    }
+    diagonal_real_array_impl(v, dest);
 }
 
-void diagonal_alloc_real_array(real_array_t* dest, int ndims, ...)
+void diagonal_alloc_real_array(const real_array_t* v, real_array_t* dest)
 {
-    size_t i;
-    size_t j;
-    va_list ap;
+  size_t n;
 
-    alloc_real_array(dest,2,ndims,ndims);
+  /* Assert that v is a vector */
+  omc_assert_macro(v->ndims == 1);
 
-    for(i = 0; i < (ndims * ndims); ++i) {
-        real_set(dest, i, 0);
-    }
-
-    va_start(ap,ndims);
-    j = 0;
-    for(i = 0; i < ndims; ++i) {
-        real_set(dest, j, va_arg(ap, modelica_real));
-        j += ndims+1;
-    }
-    va_end(ap);
+  /* Allocate a n*n matrix and fill it. */
+  n = v->dim_size[0];
+  alloc_real_array(dest, 2, n, n);
+  diagonal_real_array_impl(v, dest);
 }
 
 void fill_real_array(real_array_t* dest,modelica_real s)
