@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -48,8 +48,7 @@ OMVisualBase::OMVisualBase(const std::string& modelFile, const std::string& path
   : _shapes(),
     _modelFile(modelFile),
     _path(path),
-    _xmlFileName(assembleXMLFileName(modelFile, path)),
-    _xmlDoc()
+    _xmlFileName(assembleXMLFileName(modelFile, path))
 {
 }
 
@@ -89,35 +88,30 @@ int OMVisualBase::getShapeObjectIndexByID(std::string shapeID)
   return -1;
 }
 
-void OMVisualBase::initXMLDoc()
-{
-  // Check if the XML file is available.
-  if (!fileExists(_xmlFileName))
-  {
-    std::string msg = "Could not find the visual XML file" + _xmlFileName + ".";
-    std::cout<<msg<<std::endl;
-  }
-  // read xml
-  osgDB::ifstream t;
-  // unused const char * titel = _xmlFileName.c_str();
-  t.open(_xmlFileName.c_str(), std::ios::binary);      // open input file
-  t.seekg(0, std::ios::end);    // go to the end
-  int length = t.tellg();       // report location (this is the length)
-  t.seekg(0, std::ios::beg);    // go back to the beginning
-  char* buffer = new char[length];    // allocate memory for a buffer of appropriate dimension
-  t.read(buffer, length);       // read the whole file into the buffer
-  t.close();
-  std::string buff = std::string(buffer);  // strings are good
-  std::string buff2 = buff.substr(0, buff.find("</visualization>"));  // remove the crappy ending
-  buff2.append("</visualization>");
-  char* buff3 = strdup(buff2.c_str());  // cast to char*
-  _xmlDoc.parse<0>(buff3);
-
-}
-
 void OMVisualBase::initVisObjects()
 {
-  rapidxml::xml_node<>* rootNode = _xmlDoc.first_node();
+  if (!fileExists(_xmlFileName)) {
+    MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica,
+                                                          QString(QObject::tr("Could not find the visual XML file %1."))
+                                                          .arg(_xmlFileName.c_str()),
+                                                          Helper::scriptingKind, Helper::errorLevel));
+    return;
+  }
+
+  QFile file(QString::fromStdString(_xmlFileName));
+  if (!file.open(QIODevice::ReadOnly)) {
+    MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica,
+                                                          QString(QObject::tr("Could not open the visual XML file %1."))
+                                                          .arg(_xmlFileName.c_str()),
+                                                          Helper::scriptingKind, Helper::errorLevel));
+    return;
+  }
+  QByteArray buffer = file.readAll();
+  file.close();
+  rapidxml::xml_document<> xmlDoc;
+  xmlDoc.parse<0>(buffer.data());
+
+  rapidxml::xml_node<>* rootNode = xmlDoc.first_node();
   ShapeObject shape;
   rapidxml::xml_node<>* expNode;
 
@@ -130,7 +124,10 @@ void OMVisualBase::initVisObjects()
 
     if (expNode == 0)
     {
-      std::cout<<"The type of  "<<shape._id<<" is not supported right in the visxml file."<<std::endl;
+      MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica,
+                                                            QString(QObject::tr("The type of %1 is not supported right in the visxml file."))
+                                                            .arg(shape._id.c_str()),
+                                                            Helper::scriptingKind, Helper::errorLevel));
     }
     else
     {
@@ -150,7 +147,9 @@ void OMVisualBase::initVisObjects()
 
         if (!fileExists(shape._fileName))
         {
-          std::cout<<"Could not find the file "<<shape._fileName<<std::endl;
+          MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica,
+                                                                QString(QObject::tr("Could not find the file %1.")).arg(shape._fileName.c_str()),
+                                                                Helper::scriptingKind, Helper::errorLevel));
         }
       }
       //std::cout<<"type "<<shape._id<<std::endl;
@@ -228,16 +227,6 @@ void OMVisualBase::initVisObjects()
   } // end for-loop
 }
 
-void OMVisualBase::clearXMLDoc()
-{
-  _xmlDoc.clear();
-}
-
-rapidxml::xml_node<>* OMVisualBase::getFirstXMLNode() const
-{
-  return _xmlDoc.first_node();
-}
-
 const std::string OMVisualBase::getModelFile() const
 {
   return _modelFile;
@@ -289,10 +278,6 @@ VisualizerAbstract::VisualizerAbstract(const std::string& modelFile, const std::
 
 void VisualizerAbstract::initData()
 {
-  // In case of reloading, we need to make sure, that we have empty members.
-  mpOMVisualBase->clearXMLDoc();
-  // Initialize XML file and get visAttributes.
-  mpOMVisualBase->initXMLDoc();
   mpOMVisualBase->initVisObjects();
 }
 
@@ -372,12 +357,13 @@ std::string VisualizerAbstract::getModelFile() const
 
 void VisualizerAbstract::startVisualization()
 {
-  if (mpTimeManager->getVisTime() < mpTimeManager->getEndTime() - 1.e-6)
-  {
+  if (mpTimeManager->getVisTime() < mpTimeManager->getEndTime() - 1.e-6) {
     mpTimeManager->setPause(false);
+  } else {
+    MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica,
+                                                          QObject::tr("There is nothing left to visualize. Initialize the model first."),
+                                                          Helper::scriptingKind, Helper::errorLevel));
   }
-  else
-    std::cout<<"There is nothing left to visualize. Initialize the model first."<<std::endl;
 }
 
 void VisualizerAbstract::pauseVisualization()
@@ -538,7 +524,9 @@ void UpdateVisitor::apply(osg::Geode& node)
     }
     else
     {
-      std::cout<<"Unknown type "<<_shape._type<<", we make a capsule."<<std::endl;
+      MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica,
+                                                            QString(QObject::tr("Unknown type %1, we make a capsule.")).arg(_shape._type.c_str()),
+                                                            Helper::scriptingKind, Helper::errorLevel));
       draw->setShape(new osg::Capsule(osg::Vec3f(0.0, 0.0, 0.0), 0.1, 0.5));
     }
     //std::cout<<"SHAPE "<<draw->getShape()->className()<<std::endl;
