@@ -36,7 +36,7 @@
 #include "Modeling/Commands.h"
 
 PolygonAnnotation::PolygonAnnotation(QString annotation, GraphicsView *pGraphicsView)
-  : ShapeAnnotation(false, pGraphicsView, 0)
+  : ShapeAnnotation(false, pGraphicsView, 0, 0)
 {
   // set the default values
   GraphicItem::setDefaults();
@@ -49,32 +49,23 @@ PolygonAnnotation::PolygonAnnotation(QString annotation, GraphicsView *pGraphics
 }
 
 PolygonAnnotation::PolygonAnnotation(ShapeAnnotation *pShapeAnnotation, Component *pParent)
-  : ShapeAnnotation(pParent)
+  : ShapeAnnotation(pShapeAnnotation, pParent)
 {
   updateShape(pShapeAnnotation);
-  initUpdateVisible(); // DynamicSelect for visible attribute
   setPos(mOrigin);
   setRotation(mRotation);
-  connect(pShapeAnnotation, SIGNAL(updateReferenceShapes()), pShapeAnnotation, SIGNAL(changed()));
-  connect(pShapeAnnotation, SIGNAL(added()), this, SLOT(referenceShapeAdded()));
-  connect(pShapeAnnotation, SIGNAL(changed()), this, SLOT(referenceShapeChanged()));
-  connect(pShapeAnnotation, SIGNAL(deleted()), this, SLOT(referenceShapeDeleted()));
 }
 
 PolygonAnnotation::PolygonAnnotation(ShapeAnnotation *pShapeAnnotation, GraphicsView *pGraphicsView)
-  : ShapeAnnotation(true, pGraphicsView, 0)
+  : ShapeAnnotation(true, pGraphicsView, pShapeAnnotation, 0)
 {
   updateShape(pShapeAnnotation);
   setShapeFlags(true);
   mpGraphicsView->addItem(this);
-  connect(pShapeAnnotation, SIGNAL(updateReferenceShapes()), pShapeAnnotation, SIGNAL(changed()));
-  connect(pShapeAnnotation, SIGNAL(added()), this, SLOT(referenceShapeAdded()));
-  connect(pShapeAnnotation, SIGNAL(changed()), this, SLOT(referenceShapeChanged()));
-  connect(pShapeAnnotation, SIGNAL(deleted()), this, SLOT(referenceShapeDeleted()));
 }
 
 PolygonAnnotation::PolygonAnnotation(Component *pParent)
-  : ShapeAnnotation(pParent)
+  : ShapeAnnotation(0, pParent)
 {
   // set the default values
   GraphicItem::setDefaults();
@@ -180,6 +171,10 @@ void PolygonAnnotation::paint(QPainter *painter, const QStyleOptionGraphicsItem
   Q_UNUSED(option);
   Q_UNUSED(widget);
   if (mVisible || !mDynamicVisible.isEmpty()) {
+    if (!mDynamicVisibleValue && ((mpGraphicsView && mpGraphicsView->isVisualizationView())
+                                  || (mpParentComponent && mpParentComponent->getGraphicsView()->isVisualizationView()))) {
+      return;
+    }
     drawPolygonAnnotaion(painter);
   }
 }
@@ -222,6 +217,16 @@ QString PolygonAnnotation::getOMCShapeAnnotation()
   return annotationString.join(",");
 }
 
+/*!
+ * \brief PolygonAnnotation::getOMCShapeAnnotationWithShapeName
+ * Returns Polygon annotation in format as returned by OMC wrapped in Polygon keyword.
+ * \return
+ */
+QString PolygonAnnotation::getOMCShapeAnnotationWithShapeName()
+{
+  return QString("Polygon(%1)").arg(getOMCShapeAnnotation());
+}
+
 /*!
  * \brief PolygonAnnotation::getShapeAnnotation
  * Returns Polygon annotation.
