@@ -95,36 +95,11 @@ Parameter::Parameter(Component *pComponent, bool showStartAttribute, QString tab
   setSaveSelectorFilter("-");
   setSaveSelectorCaption("-");
   createValueWidget();
-  /* Get unit value
-   * First check if unit is defined with in the component modifier.
-   * If no unit is found then check it in the derived class modifier value.
-   * A derived class can be inherited, so look recursively.
-   */
-  QString className = mpComponent->getGraphicsView()->getModelWidget()->getLibraryTreeItem()->getNameStructure();
-  QString unit = mpComponent->getComponentInfo()->getModifiersMap(pOMCProxy, className, mpComponent).value("unit");
-  if (unit.isEmpty()) {
-    if (!pOMCProxy->isBuiltinType(mpComponent->getComponentInfo()->getClassName())) {
-      unit = pOMCProxy->getDerivedClassModifierValue(mpComponent->getComponentInfo()->getClassName(), "unit");
-      if (unit.isEmpty()) {
-        unit = getModifierValueFromDerivedClass(mpComponent, "unit");
-      }
-    }
-  }
-  mUnit = StringHandler::removeFirstLastQuotes(unit);
-  /* Get displayUnit value
-   * First check if displayUnit is defined with in the component modifier.
-   * If no displayUnit is found then check it in the derived class modifier value.
-   * A derived class can be inherited, so look recursively.
-   */
-  QString displayUnit = mpComponent->getComponentInfo()->getModifiersMap(pOMCProxy, className, mpComponent).value("displayUnit");
-  if (displayUnit.isEmpty()) {
-    if (!pOMCProxy->isBuiltinType(mpComponent->getComponentInfo()->getClassName())) {
-      displayUnit = pOMCProxy->getDerivedClassModifierValue(mpComponent->getComponentInfo()->getClassName(), "displayUnit");
-      if (displayUnit.isEmpty()) {
-        displayUnit = getModifierValueFromDerivedClass(mpComponent, "displayUnit");
-      }
-    }
-  }
+  // Get unit value
+  QString unit = mpComponent->getDerivedClassModifierValue("unit");
+  mUnit = unit;
+  // Get displayUnit value
+  QString displayUnit = mpComponent->getDerivedClassModifierValue("displayUnit");
   if (displayUnit.isEmpty()) {
     displayUnit = unit;
   }
@@ -157,28 +132,40 @@ void Parameter::updateNameLabel()
  * \param value
  * \param defaultValue
  * \param fromUnit
- * \param valueChanged
+ * \param valueModified
  * \param adjustSize
+ * \param unitComboBoxChanged
  */
-void Parameter::setValueWidget(QString value, bool defaultValue, QString fromUnit, bool valueModified, bool adjustSize)
+void Parameter::setValueWidget(QString value, bool defaultValue, QString fromUnit, bool valueModified, bool adjustSize, bool unitComboBoxChanged)
 {
-  // convert the value to display unit
-  if (!fromUnit.isEmpty() && mpUnitComboBox->currentText().compare(fromUnit) != 0) {
-    bool ok = true;
-    qreal realValue = value.toDouble(&ok);
-    // if the modifier is a literal constant
-    if (ok) {
-      OMCProxy *pOMCProxy = MainWindow::instance()->getOMCProxy();
-      OMCInterface::convertUnits_res convertUnit = pOMCProxy->convertUnits(fromUnit, mpUnitComboBox->currentText());
-      if (convertUnit.unitsCompatible) {
-        realValue = Utilities::convertUnit(realValue, convertUnit.offset, convertUnit.scaleFactor);
-        value = QString::number(realValue);
+  /* ticket:5618 if we don't have a literal constant and array of constants
+   * then we assume its an expression and don't do any conversions.
+   * So just show the unit in the unit drop down list
+   */
+  if (!unitComboBoxChanged) {
+    enableDisableUnitComboBox(value);
+  }
+  if (Utilities::isValueLiteralConstant(value)) {
+    // convert the value to display unit
+    if (!fromUnit.isEmpty() && mpUnitComboBox->currentText().compare(fromUnit) != 0) {
+      bool ok = true;
+      qreal realValue = value.toDouble(&ok);
+      // if the modifier is a literal constant
+      if (ok) {
+        OMCProxy *pOMCProxy = MainWindow::instance()->getOMCProxy();
+        OMCInterface::convertUnits_res convertUnit = pOMCProxy->convertUnits(fromUnit, mpUnitComboBox->currentText());
+        if (convertUnit.unitsCompatible) {
+          realValue = Utilities::convertUnit(realValue, convertUnit.offset, convertUnit.scaleFactor);
+          value = QString::number(realValue);
+        }
+      } else { // if expression
+        value = Utilities::arrayExpressionUnitConversion(MainWindow::instance()->getOMCProxy(), value, fromUnit, mpUnitComboBox->currentText());
       }
-    } else { // if expression
-      value = Utilities::arrayExpressionUnitConversion(MainWindow::instance()->getOMCProxy(), value, fromUnit, mpUnitComboBox->currentText());
     }
   }
-  mDefaultValue = defaultValue;
+  if (defaultValue) {
+    mDefaultValue = value;
+  }
   QFontMetrics fm = QFontMetrics(QFont());
   switch (mValueType) {
     case Parameter::Boolean:
@@ -268,16 +255,6 @@ QString Parameter::getValue()
   }
 }
 
-/*!
- * \brief Parameter::getDefaultValue
- * Returns the default value.
- * \return
- */
-QString Parameter::getDefaultValue()
-{
-  return mDefaultValue;
-}
-
 void Parameter::setFixedState(QString fixed, bool defaultValue)
 {
   mOriginalFixedValue = fixed;
@@ -293,43 +270,6 @@ QString Parameter::getFixedState()
   return mpFixedCheckBox->tickStateString();
 }
 
-/*!
- * \brief Parameter::getModifierValueFromDerivedClass
- * Returns the modifier value by reading the derived classes.
- * \param pComponent
- * \param modifierName
- * \return the modifier value.
- */
-QString Parameter::getModifierValueFromDerivedClass(Component *pComponent, QString modifierName)
-{
-  MainWindow *pMainWindow = MainWindow::instance();
-  OMCProxy *pOMCProxy = pMainWindow->getOMCProxy();
-  QString modifierValue = "";
-  if (!pComponent->getLibraryTreeItem()->getModelWidget()) {
-    pMainWindow->getLibraryWidget()->getLibraryTreeModel()->showModelWidget(pComponent->getLibraryTreeItem(), false);
-  }
-  foreach (Component *pInheritedComponent, pComponent->getInheritedComponentsList()) {
-    /* Ticket #4031
-     * Since we use the parent ComponentInfo for inherited classes so we should not use
-     * pInheritedComponent->getComponentInfo()->getClassName() to get the name instead we should use
-     * pInheritedComponent->getLibraryTreeItem()->getNameStructure() to get the correct name of inherited class.
-     * Also don't just return after reading from first inherited class. Check recursively.
-     */
-    if (pInheritedComponent->getLibraryTreeItem() &&
-        !pOMCProxy->isBuiltinType(pInheritedComponent->getLibraryTreeItem()->getNameStructure())) {
-      modifierValue = pOMCProxy->getDerivedClassModifierValue(pInheritedComponent->getLibraryTreeItem()->getNameStructure(),
-                                                              modifierName);
-      if (modifierValue.isEmpty()) {
-        modifierValue = getModifierValueFromDerivedClass(pInheritedComponent, modifierName);
-      }
-      if (!modifierValue.isEmpty()) {
-        return modifierValue;
-      }
-    }
-  }
-  return "";
-}
-
 /*!
   Sets the input field of the parameter enable/disable.
   \param enable
@@ -383,10 +323,32 @@ void Parameter::createValueWidget()
     case Parameter::Normal:
     default:
       mpValueTextBox = new QLineEdit;
+      connect(mpValueTextBox, SIGNAL(textEdited(QString)), SLOT(valueTextBoxEdited(QString)));
       break;
   }
 }
 
+/*!
+ * \brief Parameter::enableDisableUnitComboBox
+ * Enables/disables the unit combobox.
+ * \param value
+ */
+void Parameter::enableDisableUnitComboBox(const QString &value)
+{
+  /* Enable/disable the unit combobox based on the literalConstant
+   * Set the display unit as current when value is literalConstant otherwise use unit
+   */
+  bool literalConstant = Utilities::isValueLiteralConstant(value);
+  mpUnitComboBox->setEnabled(literalConstant);
+  bool state = mpUnitComboBox->blockSignals(true);
+  int index = mpUnitComboBox->findText(literalConstant ? mDisplayUnit : mUnit, Qt::MatchExactly);
+  if (index > -1 && index != mpUnitComboBox->currentIndex()) {
+    mpUnitComboBox->setCurrentIndex(index);
+    mPreviousUnit = mpUnitComboBox->currentText();
+  }
+  mpUnitComboBox->blockSignals(state);
+}
+
 /*!
  * \brief Parameter::fileSelectorButtonClicked
  * Slot activated when mpFileSelectorButton clicked SIGNAL is raised.
@@ -434,13 +396,12 @@ void Parameter::fileSelectorButtonClicked()
  */
 void Parameter::unitComboBoxChanged(QString text)
 {
-  QString defaultValue = getDefaultValue();
-  if (!defaultValue.isEmpty()) {
-    setValueWidget(getDefaultValue(), true, mPreviousUnit, false);
+  if (!mDefaultValue.isEmpty()) {
+    setValueWidget(mDefaultValue, true, mPreviousUnit, false, true, true);
   }
   QString value = getValue();
   if (!value.isEmpty()) {
-    setValueWidget(getValue(), false, mPreviousUnit, true);
+    setValueWidget(value, false, mPreviousUnit, true, true, true);
   }
   mPreviousUnit = text;
 }
@@ -465,9 +426,28 @@ void Parameter::valueComboBoxChanged(int index)
  */
 void Parameter::valueCheckBoxChanged(bool toggle)
 {
+  Q_UNUSED(toggle);
   mValueCheckBoxModified = true;
 }
 
+/*!
+ * \brief Parameter::valueTextBoxEdited
+ * This slot is only called when user manually edits the text.\n
+ * \param text
+ */
+void Parameter::valueTextBoxEdited(const QString &text)
+{
+  // if we don't have the text we use the default value otherwise the text
+  QString value;
+  if (text.isEmpty()) {
+    value = mDefaultValue;
+  } else {
+    value = text;
+  }
+
+  enableDisableUnitComboBox(value);
+}
+
 void Parameter::showFixedMenu()
 {
   // create a menu
@@ -901,7 +881,8 @@ void ComponentParameters::createTabsGroupBoxesAndParametersHelper(LibraryTreeIte
     /* get the dialog annotation */
     QStringList dialogAnnotation = pComponent->getDialogAnnotation();
     QString groupImage = "";
-    if (dialogAnnotation.size() > 0) {
+    bool connectorSizing = false;
+    if (dialogAnnotation.size() > 10) {
       // get the tab value
       tab = StringHandler::removeFirstLastQuotes(dialogAnnotation.at(0));
       // get the group value
@@ -926,6 +907,12 @@ void ComponentParameters::createTabsGroupBoxesAndParametersHelper(LibraryTreeIte
       if (!groupImage.isEmpty()) {
         groupImage = MainWindow::instance()->getOMCProxy()->uriToFilename(groupImage);
       }
+      // get the connectorSizing
+      connectorSizing = (dialogAnnotation.at(10).compare("true") == 0);
+    }
+    // if connectorSizing is present then don't show the parameter
+    if (connectorSizing) {
+      continue;
     }
     // if showStartAttribute true and group name is empty or Parameters then we should make group name Initialization
     if (showStartAttribute && groupBox.isEmpty()) {
@@ -1216,7 +1203,7 @@ void ComponentParameters::updateComponentParameters()
       }
     }
     // if displayUnit is changed
-    if (pParameter->getDisplayUnit().compare(pParameter->getUnitComboBox()->currentText()) != 0) {
+    if (pParameter->getUnitComboBox()->isEnabled() && pParameter->getDisplayUnit().compare(pParameter->getUnitComboBox()->currentText()) != 0) {
       valueChanged = true;
       /* If the component is inherited then add the modifier value into the extends. */
       if (mpComponent->isInheritedComponent()) {
@@ -1240,7 +1227,7 @@ void ComponentParameters::updateComponentParameters()
         QString componentModifierValue = modifier.mid(modifier.indexOf("("));
         newComponentModifiersMap.insert(componentModifierKey, componentModifierValue);
       } else {
-        MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica, "", false, 0, 0, 0, 0,
+        MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica,
                                                               GUIMessages::getMessage(GUIMessages::WRONG_MODIFIER).arg(modifier),
                                                               Helper::scriptingKind, Helper::errorLevel));
       }
@@ -1262,8 +1249,6 @@ void ComponentParameters::updateComponentParameters()
                                                                              newComponentExtendsModifiersMap);
     mpComponent->getGraphicsView()->getModelWidget()->getUndoStack()->push(pUpdateComponentParametersCommand);
     mpComponent->getGraphicsView()->getModelWidget()->updateModelText();
-    // remove possible dynamic results to see the new static values
-    mpComponent->getGraphicsView()->getModelWidget()->removeDynamicResults();
   }
   accept();
 }
@@ -1407,7 +1392,7 @@ void ComponentAttributes::initializeDialog()
   mpNameTextBox->setText(mpComponent->getComponentInfo()->getName());
   mpNameTextBox->setCursorPosition(0);
   // get dimensions
-  QString dimensions = StringHandler::removeFirstLastCurlBrackets(mpComponent->getComponentInfo()->getArrayIndex());
+  QString dimensions = mpComponent->getComponentInfo()->getArrayIndex();
   mpDimensionsTextBox->setText(QString("[%1]").arg(dimensions));
   // get Comment
   mpCommentTextBox->setText(mpComponent->getComponentInfo()->getComment());
@@ -1519,8 +1504,7 @@ void ComponentAttributes::updateComponentAttributes()
   /* If user has really changed the Component's attributes then push that change on the stack.
    */
   if (oldComponentInfo != newComponentInfo) {
-    UpdateComponentAttributesCommand *pUpdateComponentAttributesCommand = new UpdateComponentAttributesCommand(mpComponent, oldComponentInfo,
-                                                                                                               newComponentInfo);
+    UpdateComponentAttributesCommand *pUpdateComponentAttributesCommand = new UpdateComponentAttributesCommand(mpComponent, oldComponentInfo, newComponentInfo);
     pModelWidget->getUndoStack()->push(pUpdateComponentAttributesCommand);
     pModelWidget->updateModelText();
   }
