@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -29,21 +29,17 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- * RCS: $Id$
- *
  */
 
 #ifndef CORNERITEM_H
 #define CORNERITEM_H
 
+#include "Annotations/ShapeAnnotation.h"
 #include <QObject>
 #include <QGraphicsItem>
 #include <QPen>
 #include <QGraphicsSceneMouseEvent>
-#include "ShapeAnnotation.h"
 
 class CornerItem : public QObject, public QGraphicsItem
 {
@@ -52,23 +48,24 @@ class CornerItem : public QObject, public QGraphicsItem
 private:
   ShapeAnnotation *mpShapeAnnotation;
   QRectF mRectangle;
+  QString mOldAnnotation;
   QPointF mClickPos;
   int mConnectedPointIndex;
 public:
   CornerItem(qreal x, qreal y, int connectedPointIndex, ShapeAnnotation *pParent);
   void setConnectedPointIndex(int connectedPointIndex) {mConnectedPointIndex = connectedPointIndex;}
   int getConnectetPointIndex() {return mConnectedPointIndex;}
-  QRectF boundingRect() const;
-  void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);
+  QRectF boundingRect() const override;
+  void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0) override;
 signals:
   void cornerItemMoved(int index, QPointF point);
   void cornerItemPress();
   void cornerItemRelease();
   void cornerItemPositionChanged();
 protected:
-  virtual void mousePressEvent(QGraphicsSceneMouseEvent *event);
-  virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
-  QVariant itemChange(GraphicsItemChange change, const QVariant &value);
+  virtual void mousePressEvent(QGraphicsSceneMouseEvent *event) override;
+  virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent *event) override;
+  QVariant itemChange(GraphicsItemChange change, const QVariant &value) override;
 };
 
 class ResizerItem : public QObject, public QGraphicsItem
@@ -82,14 +79,15 @@ public:
   ResizePositions getResizePosition();
   void setActive();
   void setPassive();
-  QRectF boundingRect() const;
-  void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);
+  QRectF boundingRect() const override;
+  void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0) override;
   bool isPressed();
 private:
   Component *mpComponent;
   QRectF mRectangle;
   QPen mPen;
   QPen mActivePen;
+  QPen mInheritedActivePen;
   QPen mPassivePen;
   bool mIsPressed;
   QPointF mResizerItemOldPosition;
@@ -100,23 +98,25 @@ signals:
   void resizerItemReleased();
   void resizerItemPositionChanged();
 protected:
-  virtual void mousePressEvent(QGraphicsSceneMouseEvent *event);
-  virtual void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
-  virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
+  virtual void mousePressEvent(QGraphicsSceneMouseEvent *event) override;
+  virtual void mouseMoveEvent(QGraphicsSceneMouseEvent *event) override;
+  virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent *event) override;
 };
 
 class OriginItem : public QGraphicsItem
 {
 public:
-  OriginItem();
-  void setActive() {mPen = mActivePen;}
-  void setPassive() {mPen = mPassivePen;}
+  OriginItem(Component *pComponent);
+  void setActive();
+  void setPassive();
   QRectF boundingRect() const {return mRectangle;}
   void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);
 private:
+  Component *mpComponent;
   QRectF mRectangle;
   QPen mPen;
   QPen mActivePen;
+  QPen mInheritedActivePen;
   QPen mPassivePen;
 };
 
