@@ -70,6 +70,7 @@ class OMSimulatorPage;
 class TraceabilityPage;
 class TabSettings;
 class StackFramesWidget;
+class TranslationFlagsWidget;
 
 class OptionsDialog : public QDialog
 {
@@ -84,7 +85,11 @@ private:
 
   static OptionsDialog *mpInstance;
 public:
-  static OptionsDialog* instance() {return mpInstance;}
+  static OptionsDialog* instance() {
+    create();
+    return mpInstance;
+  }
+  ~OptionsDialog() { mpInstance = nullptr; }
   void readSettings();
   void readGeneralSettings();
   void readLibrariesSettings();
@@ -122,6 +127,7 @@ public:
   void saveTraceabilitySettings();
   void saveGraphicalViewsSettings();
   void saveSimulationSettings();
+  void saveGlobalSimulationSettings();
   void saveMessagesSettings();
   void saveNotificationsSettings();
   void saveLineStyleSettings();
@@ -171,6 +177,7 @@ public slots:
   void changePage(QListWidgetItem *current, QListWidgetItem *previous);
   void reject();
   void saveSettings();
+  void reset();
 private:
   GeneralSettingsPage *mpGeneralSettingsPage;
   LibrariesPage *mpLibrariesPage;
@@ -201,6 +208,7 @@ private:
   Label *mpChangesEffectLabel;
   QPushButton *mpCancelButton;
   QPushButton *mpOkButton;
+  QPushButton *mpResetButton;
   QDialogButtonBox *mpButtonBox;
 };
 
@@ -208,6 +216,11 @@ class GeneralSettingsPage : public QWidget
 {
   Q_OBJECT
 public:
+  enum AccessAnnotations {
+    Always = 0,
+    Loading = 1,
+    Never = 2
+  };
   GeneralSettingsPage(OptionsDialog *pOptionsDialog);
   QComboBox* getLanguageComboBox() {return mpLanguageComboBox;}
   void setWorkingDirectory(QString value) {mpWorkingDirectoryTextBox->setText(value);}
@@ -220,8 +233,10 @@ public:
   void setTerminalCommandArguments(QString value) {mpTerminalCommandArgumentsTextBox->setText(value);}
   QString getTerminalCommandArguments() {return mpTerminalCommandArgumentsTextBox->text();}
   QCheckBox* getHideVariablesBrowserCheckBox() {return mpHideVariablesBrowserCheckBox;}
-  QCheckBox* getActivateAccessAnnotationsCheckBox() {return mpActivateAccessAnnotationsCheckBox;}
+  QComboBox* getActivateAccessAnnotationsComboBox() {return mpActivateAccessAnnotationsComboBox;}
+  QCheckBox* getCreateBackupFileCheckbox() {return mpCreateBackupFileCheckbox;}
   QSpinBox* getLibraryIconSizeSpinBox() {return mpLibraryIconSizeSpinBox;}
+  QSpinBox* getLibraryIconTextLengthSpinBox() {return mpLibraryIconTextLengthSpinBox;}
   void setShowProtectedClasses(bool value) {mpShowProtectedClasses->setChecked(value);}
   bool getShowProtectedClasses() {return mpShowProtectedClasses->isChecked();}
   void setShowHiddenClasses(bool value) {mpShowHiddenClasses->setChecked(value);}
@@ -252,10 +267,14 @@ private:
   Label *mpTerminalCommandArgumentsLabel;
   QLineEdit *mpTerminalCommandArgumentsTextBox;
   QCheckBox *mpHideVariablesBrowserCheckBox;
-  QCheckBox *mpActivateAccessAnnotationsCheckBox;
+  Label *mpActivateAccessAnnotationsLabel;
+  QComboBox *mpActivateAccessAnnotationsComboBox;
+  QCheckBox *mpCreateBackupFileCheckbox;
   QGroupBox *mpLibrariesBrowserGroupBox;
   Label *mpLibraryIconSizeLabel;
   QSpinBox *mpLibraryIconSizeSpinBox;
+  Label *mpLibraryIconTextLengthLabel;
+  QSpinBox *mpLibraryIconTextLengthSpinBox;
   QCheckBox *mpShowProtectedClasses;
   QCheckBox *mpShowHiddenClasses;
   QGroupBox *mpModelingViewModeGroupBox;
@@ -594,13 +613,14 @@ class SimulationPage : public QWidget
   Q_OBJECT
 public:
   SimulationPage(OptionsDialog *pOptionsDialog);
-  QComboBox* getMatchingAlgorithmComboBox() {return mpMatchingAlgorithmComboBox;}
-  QComboBox* getIndexReductionMethodComboBox() {return mpIndexReductionMethodComboBox;}
+  TranslationFlagsWidget *getTranslationFlagsWidget() const {return mpTranslationFlagsWidget;}
   QComboBox* getTargetLanguageComboBox() {return mpTargetLanguageComboBox;}
-  QComboBox* getTargetCompilerComboBox() {return mpTargetCompilerComboBox;}
-  QLineEdit* getOMCCommandLineOptionsTextBox() {return mpOMCCommandLineOptionsTextBox;}
+  QComboBox* getTargetBuildComboBox() {return mpTargetBuildComboBox;}
+  QComboBox* getCompilerComboBox() {return mpCompilerComboBox;}
+  QComboBox* getCXXCompilerComboBox() {return mpCXXCompilerComboBox;}
   QCheckBox* getIgnoreCommandLineOptionsAnnotationCheckBox() {return mpIgnoreCommandLineOptionsAnnotationCheckBox;}
   QCheckBox* getIgnoreSimulationFlagsAnnotationCheckBox() {return mpIgnoreSimulationFlagsAnnotationCheckBox;}
+  QCheckBox* getEnableNewInstantiationAPICheckBox() {return mpEnableNewInstantiationAPICheckBox;}
   QCheckBox* getSaveClassBeforeSimulationCheckBox() {return mpSaveClassBeforeSimulationCheckBox;}
   QCheckBox* getSwitchToPlottingPerspectiveCheckBox() {return mpSwitchToPlottingPerspectiveCheckBox;}
   QCheckBox* getCloseSimulationOutputWidgetsBeforeSimulationCheckBox() {return mpCloseSimulationOutputWidgetsBeforeSimulationCheckBox;}
@@ -608,22 +628,23 @@ public:
   QCheckBox* getDeleteEntireSimulationDirectoryCheckBox() {return mpDeleteEntireSimulationDirectoryCheckBox;}
   void setOutputMode(QString value);
   QString getOutputMode();
+  QSpinBox* getDisplayLimitSpinBox() {return mpDisplayLimitSpinBox;}
 private:
   OptionsDialog *mpOptionsDialog;
   QGroupBox *mpSimulationGroupBox;
-  Label *mpMatchingAlgorithmLabel;
-  QComboBox *mpMatchingAlgorithmComboBox;
-  Label *mpIndexReductionMethodLabel;
-  QComboBox *mpIndexReductionMethodComboBox;
+  QGroupBox *mpTranslationFlagsGroupBox;
+  TranslationFlagsWidget *mpTranslationFlagsWidget;
   Label *mpTargetLanguageLabel;
   QComboBox *mpTargetLanguageComboBox;
+  Label *mpTargetBuildLabel;
+  QComboBox *mpTargetBuildComboBox;
   Label *mpCompilerLabel;
-  QComboBox *mpTargetCompilerComboBox;
-  Label *mpOMCCommandLineOptionsLabel;
-  QLineEdit *mpOMCCommandLineOptionsTextBox;
-  QToolButton *mpOMCCommandLineOptionsHelpButton;
+  QComboBox *mpCompilerComboBox;
+  Label *mpCXXCompilerLabel;
+  QComboBox *mpCXXCompilerComboBox;
   QCheckBox *mpIgnoreCommandLineOptionsAnnotationCheckBox;
   QCheckBox *mpIgnoreSimulationFlagsAnnotationCheckBox;
+  QCheckBox *mpEnableNewInstantiationAPICheckBox;
   QCheckBox *mpSaveClassBeforeSimulationCheckBox;
   QCheckBox *mpSwitchToPlottingPerspectiveCheckBox;
   QCheckBox *mpCloseSimulationOutputWidgetsBeforeSimulationCheckBox;
@@ -632,10 +653,12 @@ private:
   QGroupBox *mpOutputGroupBox;
   QRadioButton *mpStructuredRadioButton;
   QRadioButton *mpFormattedTextRadioButton;
+  Label *mpDisplayLimitLabel;
+  QSpinBox *mpDisplayLimitSpinBox;
+  Label *mpDisplayLimitMBLabel;
 public slots:
-  void updateMatchingAlgorithmToolTip(int index);
-  void updateIndexReductionToolTip(int index);
-  void showOMCCommandLineOptionsHelp();
+  void targetBuildChanged(int index);
+  void displayLimitValueChanged(int value);
 };
 
 class MessagesPage : public QWidget
@@ -688,6 +711,12 @@ class NotificationsPage : public QWidget
 {
   Q_OBJECT
 public:
+  enum OldFrontend {
+    AlwaysAskForOF = 0,
+    TryOnceWithOF = 1,
+    SwitchPermanentlyToOF = 2,
+    KeepUsingNF = 3
+  };
   NotificationsPage(OptionsDialog *pOptionsDialog);
   QCheckBox* getQuitApplicationCheckBox() {return mpQuitApplicationCheckBox;}
   QCheckBox* getItemDroppedOnItselfCheckBox() {return mpItemDroppedOnItselfCheckBox;}
@@ -696,6 +725,7 @@ public:
   QCheckBox* getSaveModelForBitmapInsertionCheckBox() {return mpSaveModelForBitmapInsertionCheckBox;}
   QCheckBox* getAlwaysAskForDraggedComponentName() {return mpAlwaysAskForDraggedComponentName;}
   QCheckBox* getAlwaysAskForTextEditorErrorCheckBox() {return mpAlwaysAskForTextEditorErrorCheckBox;}
+  QComboBox* getOldFrontendComboBox() {return mpOldFrontendComboBox;}
 private:
   OptionsDialog *mpOptionsDialog;
   QGroupBox *mpNotificationsGroupBox;
@@ -706,6 +736,8 @@ private:
   QCheckBox *mpSaveModelForBitmapInsertionCheckBox;
   QCheckBox *mpAlwaysAskForDraggedComponentName;
   QCheckBox *mpAlwaysAskForTextEditorErrorCheckBox;
+  Label *mpOldFrontendLabel;
+  QComboBox *mpOldFrontendComboBox;
 };
 
 class LineStylePage : public QWidget
@@ -784,6 +816,14 @@ public:
   int getCurvePattern();
   void setCurveThickness(qreal thickness);
   qreal getCurveThickness();
+  QSpinBox* getFilterIntervalSpinBox() {return mpFilterIntervalSpinBox;}
+  DoubleSpinBox *getTitleFontSizeSpinBox() const {return mpTitleFontSizeSpinBox;}
+  DoubleSpinBox *getVerticalAxisTitleFontSizeSpinBox() const {return mpVerticalAxisTitleFontSizeSpinBox;}
+  DoubleSpinBox *getVerticalAxisNumbersFontSizeSpinBox() const {return mpVerticalAxisNumbersFontSizeSpinBox;}
+  DoubleSpinBox *getHorizontalAxisTitleFontSizeSpinBox() const {return mpHorizontalAxisTitleFontSizeSpinBox;}
+  DoubleSpinBox *getHorizontalAxisNumbersFontSizeSpinBox() const {return mpHorizontalAxisNumbersFontSizeSpinBox;}
+  DoubleSpinBox *getFooterFontSizeSpinBox() const {return mpFooterFontSizeSpinBox;}
+  DoubleSpinBox *getLegendFontSizeSpinBox() const {return mpLegendFontSizeSpinBox;}
 private:
   OptionsDialog *mpOptionsDialog;
   QGroupBox *mpGeneralGroupBox;
@@ -796,6 +836,25 @@ private:
   QComboBox *mpCurvePatternComboBox;
   Label *mpCurveThicknessLabel;
   DoubleSpinBox *mpCurveThicknessSpinBox;
+  QGroupBox *mpVariableFilterGroupBox;
+  Label *mpFilterIntervalHelpLabel;
+  Label *mpFilterIntervalLabel;
+  QSpinBox *mpFilterIntervalSpinBox;
+  QGroupBox *mpFontSizeGroupBox;
+  Label *mpTitleFontSizeLabel;
+  DoubleSpinBox *mpTitleFontSizeSpinBox;
+  Label *mpVerticalAxisTitleFontSizeLabel;
+  DoubleSpinBox *mpVerticalAxisTitleFontSizeSpinBox;
+  Label *mpVerticalAxisNumbersFontSizeLabel;
+  DoubleSpinBox *mpVerticalAxisNumbersFontSizeSpinBox;
+  Label *mpHorizontalAxisTitleFontSizeLabel;
+  DoubleSpinBox *mpHorizontalAxisTitleFontSizeSpinBox;
+  Label *mpHorizontalAxisNumbersFontSizeLabel;
+  DoubleSpinBox *mpHorizontalAxisNumbersFontSizeSpinBox;
+  Label *mpFooterFontSizeLabel;
+  DoubleSpinBox *mpFooterFontSizeSpinBox;
+  Label *mpLegendFontSizeLabel;
+  DoubleSpinBox *mpLegendFontSizeSpinBox;
 };
 
 class FigaroPage : public QWidget
@@ -874,9 +933,14 @@ public:
   void setFMIExportType(QString type);
   QString getFMIExportType();
   QLineEdit* getFMUNameTextBox() {return mpFMUNameTextBox;}
+  QLineEdit* getMoveFMUTextBox() {return mpMoveFMUTextBox;}
   QGroupBox* getPlatformsGroupBox() {return mpPlatformsGroupBox;}
   QComboBox* getLinkingComboBox() {return mpLinkingComboBox;}
   QCheckBox* getDeleteFMUDirectoryAndModelCheckBox() {return mpDeleteFMUDirectoryAndModelCheckBox;}
+
+  static const QString FMU_FULL_CLASS_NAME_DOTS_PLACEHOLDER;
+  static const QString FMU_FULL_CLASS_NAME_UNDERSCORES_PLACEHOLDER;
+  static const QString FMU_SHORT_CLASS_NAME_PLACEHOLDER;
 private:
   OptionsDialog *mpOptionsDialog;
   QGroupBox *mpExportGroupBox;
@@ -888,11 +952,16 @@ private:
   QRadioButton *mpCoSimulationRadioButton;
   QRadioButton *mpModelExchangeCoSimulationRadioButton;
   Label *mpFMUNameLabel;
+  Label *mpMoveFMULabel;
   QLineEdit *mpFMUNameTextBox;
+  QLineEdit *mpMoveFMUTextBox;
+  QPushButton *mpBrowseFMUDirectoryButton;
   QGroupBox *mpPlatformsGroupBox;
   QComboBox *mpLinkingComboBox;
   QGroupBox *mpImportGroupBox;
   QCheckBox *mpDeleteFMUDirectoryAndModelCheckBox;
+public slots:
+  void selectFMUDirectory();
 };
 
 class TLMPage : public QWidget
