@@ -55,6 +55,7 @@ QString Helper::fmuFileTypes = "FMU Files (*.fmu)";
 QString Helper::xmlFileTypes = "XML Files (*.xml)";
 QString Helper::infoXmlFileTypes = "OM Info Files (*_info.json)";
 QString Helper::matFileTypes = "MAT Files (*.mat)";
+QString Helper::csvFileTypes = "CSV Files (*.csv)";
 QString Helper::omResultFileTypes = "OpenModelica Result Files (*.mat *.plt *.csv)";
 #ifdef WIN32
 QString Helper::exeFileTypes = "EXE Files (*.exe)";
@@ -69,9 +70,11 @@ QString Helper::subModelFileTypes = "SubModel Files (*.fmu *.mat *.csv);;SubMode
 int Helper::treeIndentation = 13;
 QSize Helper::iconSize = QSize(20, 20);
 int Helper::tabWidth = 20;
+qreal Helper::minimumTextFontSize = 8.0;
 QString Helper::modelicaComponentFormat = "image/modelica-component";
 QString Helper::modelicaFileFormat = "text/uri-list";
 QString Helper::busConnectorFormat = "bus/connector";
+QString Helper::cutCopyPasteFormat = "application/OMEdit.cut-copy-paste";
 qreal Helper::shapesStrokeWidth = 2.0;
 int Helper::headingFontSize = 18;
 QString Helper::ModelicaSimulationOutputFormats = "mat,plt,csv";
@@ -157,6 +160,7 @@ QString Helper::output;
 QString Helper::parameters;
 QString Helper::inputs;
 QString Helper::name;
+QString Helper::startScript;
 QString Helper::comment;
 QString Helper::path;
 QString Helper::type;
@@ -170,10 +174,13 @@ QString Helper::checkAllModels;
 QString Helper::checkAllModelsTip;
 QString Helper::instantiateModel;
 QString Helper::instantiateModelTip;
-QString Helper::exportFMU;
+QString Helper::FMU;
+QString Helper::exportt;
 QString Helper::exportFMUTip;
 QString Helper::exportEncryptedPackage;
 QString Helper::exportEncryptedPackageTip;
+QString Helper::exportReadonlyPackage;
+QString Helper::exportRealonlyPackageTip;
 QString Helper::importFMU;
 QString Helper::importFMUTip;
 QString Helper::exportXML;
@@ -381,10 +388,10 @@ QString Helper::moveUp;
 QString Helper::moveDown;
 QString Helper::fixErrorsManually;
 QString Helper::revertToLastCorrectVersion;
-QString Helper::OMCCommandLineOptions;
-QString Helper::OMCCommandLineOptionsTip;
+QString Helper::translationFlagsTip;
 QString Helper::saveExperimentAnnotation;
 QString Helper::saveOpenModelicaSimulationFlagsAnnotation;
+QString Helper::saveOpenModelicaCommandLineOptionsAnnotation;
 QString Helper::item;
 QString Helper::bold;
 QString Helper::italic;
@@ -406,6 +413,7 @@ QString Helper::terminateInstantiation;
 QString Helper::terminateInstantiationTip;
 QString Helper::archivedSimulations;
 QString Helper::systemSimulationInformation;
+QString Helper::translationFlags;
 
 void Helper::initHelperVariables()
 {
@@ -446,6 +454,7 @@ void Helper::initHelperVariables()
   Helper::parameters = tr("Parameters");
   Helper::inputs = tr("Inputs");
   Helper::name = tr("Name:");
+  Helper::startScript = tr("Start Script:");
   Helper::comment = tr("Comment:");
   Helper::path = tr("Path:");
   Helper::type = tr("Type");
@@ -459,13 +468,16 @@ void Helper::initHelperVariables()
   Helper::checkAllModelsTip = tr("Checks all nested modelica classes");
   Helper::instantiateModel = tr("Instantiate Model");
   Helper::instantiateModelTip = tr("Instantiate/Flatten the Modelica class");
-  Helper::exportFMU = tr("Export FMU");
+  Helper::FMU = tr("FMU");
+  Helper::exportt = tr("Export");
   Helper::exportFMUTip = tr("Exports the model as Functional Mockup Unit (FMU)");
-  Helper::exportEncryptedPackage = tr("Export Encrypted Package");
+  Helper::exportReadonlyPackage = tr("Read-only Package");
+  Helper::exportRealonlyPackageTip = tr("Exports the package as read-only package");
+  Helper::exportEncryptedPackage = tr("Encrypted Package");
   Helper::exportEncryptedPackageTip = tr("Exports the package as Encrytped package");
   Helper::importFMU = tr("Import FMU");
   Helper::importFMUTip = tr("Imports the model from Functional Mockup Interface (FMU)");
-  Helper::exportXML = tr("Export XML");
+  Helper::exportXML = tr("XML");
   Helper::exportXMLTip = tr("Exports the model as XML");
   Helper::exportToOMNotebook = tr("Export to OMNotebook");
   Helper::exportToOMNotebookTip = tr("Exports the current model to OMNotebook");
@@ -670,10 +682,10 @@ void Helper::initHelperVariables()
   Helper::moveDown = tr("Move Down");
   Helper::fixErrorsManually = tr("Fix error(s) manually");
   Helper::revertToLastCorrectVersion = tr("Revert to last correct version");
-  Helper::OMCCommandLineOptions = tr("OMC Command Line Options");
-  Helper::OMCCommandLineOptionsTip = tr("Space separated list of OMC command line options e.g., -d=initialization --cheapmatchingAlgorithm=3");
-  Helper::saveExperimentAnnotation = tr("Save experiment annotation inside model");
-  Helper::saveOpenModelicaSimulationFlagsAnnotation = tr("Save __OpenModelica_simulationFlags annotation inside model");
+  Helper::translationFlagsTip = tr("Space separated list of OMC command line options e.g., -d=initialization --cheapmatchingAlgorithm=3");
+  Helper::saveExperimentAnnotation = tr("Save experiment annotation inside model i.e., experiment annotation");
+  Helper::saveOpenModelicaSimulationFlagsAnnotation = tr("Save simulation flags inside model i.e., __OpenModelica_simulationFlags annotation");
+  Helper::saveOpenModelicaCommandLineOptionsAnnotation = tr("Save translation flags inside model i.e., __OpenModelica_commandLineOptions annotation");
   Helper::item = tr("item");
   Helper::bold = tr("Bold");
   Helper::italic = tr("Italic");
@@ -695,6 +707,7 @@ void Helper::initHelperVariables()
   Helper::terminateInstantiationTip = tr("Terminates the model instantiation");
   Helper::archivedSimulations = tr("Archived Simulations");
   Helper::systemSimulationInformation = tr("System Simulation Information");
+  Helper::translationFlags = tr("Translation Flags");
 }
 
 QString GUIMessages::getMessage(int type)
@@ -704,7 +717,7 @@ QString GUIMessages::getMessage(int type)
     case CHECK_MESSAGES_BROWSER:
       return tr("Please check the Messages Browser for more error specific details.");
     case SAME_COMPONENT_NAME:
-      return tr("A component with the name <b>%1</b> already exists. Please choose another name.");
+      return tr("A component with the name <b>%1</b> already exists or is a Modelica keyword. Please choose another name.");
     case SAME_COMPONENT_CONNECT:
       return tr("You cannot connect a component to itself.");
     case NO_MODELICA_CLASS_OPEN:
@@ -733,6 +746,8 @@ QString GUIMessages::getMessage(int type)
       return tr("<br /><br />If you cannot find the source of the error, you can always <b>revert to the last correct version</b>.");
     case NO_OPENMODELICA_KEYWORDS:
       return tr("Please make sure you are not using any OpenModelica Keywords like (model, package, record, class etc.)");
+    case UNABLE_TO_CREATE_CLASS:
+      return tr("Unable to create class <b>%1</b>. Invalid name.<br /><br />%2.");
     case UNABLE_TO_LOAD_FILE:
       return tr("Error has occurred while loading the file/library <b>%1</b>. Unable to load the file/library.");
     case UNABLE_TO_OPEN_FILE:
@@ -752,7 +767,9 @@ QString GUIMessages::getMessage(int type)
     case REDEFINING_EXISTING_CLASSES:
       return tr("Redefining class(es) <b>%1</b> which already exist(s).");
     case MULTIPLE_TOP_LEVEL_CLASSES:
-      return tr("Only single nonstructured entity is allowed to be stored in the file. <b>%1</b> contains following classes <b>%2</b>.");
+      return tr("Only single nonstructured entity is allowed to be stored in the file.<br />"
+                "If the file was generated by the API function <b>saveTotalModel()</b>, you can only load it with the API function <b>loadFile()</b> in a script or in the interactive environment, not in OMEdit."
+                "<br /><br /><b>%1</b> contains following classes <b>%2</b>.");
     case DIAGRAM_VIEW_DROP_MSG:
       return tr("You cannot insert <b>%1</b>, it is a <b>%2</b>. Only <b>model</b>, <b>class</b>, <b>connector</b>, <b>record</b> or <b>block</b> is allowed on the diagram layer.");
     case ICON_VIEW_DROP_MSG:
@@ -771,6 +788,8 @@ QString GUIMessages::getMessage(int type)
       return tr("A component with the name <b>%1</b> already exists. The name is changed from <b>%1</b> to <b>%2</b>.<br /><br />This is probably wrong because the component is declared as <b>inner</b>.");
     case FMU_GENERATED:
       return tr("The FMU is generated at <b>%1</b>.");
+    case FMU_MOVE_FAILED:
+      return tr("Cannot move FMU to <b>%1</b>.");
     case FMU_EMPTY_PLATFORMS:
       return tr("A source-only FMU will be generated because an empty list of platforms is selected. If this is not intended, check settings in <b>%1->FMI->Platforms</b>.");
     case XML_GENERATED:
@@ -779,6 +798,8 @@ QString GUIMessages::getMessage(int type)
       return tr("The FIGARO is generated.");
     case ENCRYPTED_PACKAGE_GENERATED:
       return tr("The encrytped package is generated at <b>%1</b>.");
+    case READONLY_PACKAGE_GENERATED:
+      return tr("The read-only package is generated at <b>%1</b>.");
     case UNLOAD_CLASS_MSG:
       return tr("Are you sure you want to unload <b>%1</b>? Everything contained inside this class will also be unloaded.");
     case DELETE_CLASS_MSG:
@@ -820,8 +841,9 @@ QString GUIMessages::getMessage(int type)
     case UNABLE_FIND_COMPONENT_IN_INITIALSTATE:
       return tr("Unable to find component %1 while parsing initialState(%2).");
     case SELECT_SIMULATION_OPTION:
-      return tr("Select at least one of the following options, <br /><br />* %1<br />* %2<br />* %3")
+      return tr("Select at least one of the following options, <br /><br />* %1<br />* %2<br />* %3<br />* %4")
           .arg(Helper::saveExperimentAnnotation)
+          .arg(Helper::saveOpenModelicaCommandLineOptionsAnnotation)
           .arg(Helper::saveOpenModelicaSimulationFlagsAnnotation)
           .arg(Helper::simulate);
     case INVALID_TRANSITION_CONDITION:
@@ -832,6 +854,8 @@ QString GUIMessages::getMessage(int type)
       return tr("Following error has occurred <b>%1</b> GDB arguments are <b>\"%2\"</b>");
     case INVALID_INSTANCE_NAME:
       return tr("Name <b>%1</b> is not a valid identifier.<br />A name must start with a letter, and all characters must be letters or digits. It may not be a reserved word.");
+    case ENTER_SCRIPT:
+      return tr("Please enter a script file.");
     default:
       return "";
   }
