@@ -155,11 +155,15 @@ public:
   static QColor getSimulationMessageTypeColor(StringHandler::SimulationMessageType type);
   static QString makeClassNameRelative(QString draggedClassName, QString droppedClassName);
   static QString toCamelCase(QString str);
-  static QMap<int, int> getLeadingSpaces(QString contents, int tabWidth);
+  static QMap<int, int> getLeadingSpaces(QString contents);
   static int getLeadingSpacesSize(QString str);
   static bool isFileWritAble(QString filePath);
   static bool containsSpace(QString str);
+  static QString trimmedEnd(const QString &str);
   static QString joinDerivativeAndPreviousVariable(QString fullVariableName, QString variableName, QString derivativeOrPrevious);
+  static QString removeLeadingSpaces(QString contents);
+  static QString removeLine(QString text, QString lineToRemove);
+  static QString insertClassAtPosition(QString parentClassText, QString childClassText, int linePosition, int nestedLevel);
 protected:
   static QString mLastOpenDir;
 };
