@@ -21,7 +21,7 @@
  * GNU version 3 is obtained from: http://www.gnu.org/copyleft/gpl.html.
  *
  * This program is distributed WITHOUT ANY WARRANTY; without
- * even the implied warranty of  MERCHANTABILITY or FITNESS
+ * even the implied warranty of MERCHANTABILITY or FITNESS
  * FOR A PARTICULAR PURPOSE, EXCEPT AS EXPRESSLY SET FORTH
  * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS OF OSMC-PL.
  *
@@ -39,7 +39,7 @@
 #include "SystemSC.h"
 #include <fmilib.h>
 #include <JM/jm_portability.h>
-#include <OMSBoost.h>
+#include <OMSFileSystem.h>
 #include <RegEx.h>
 
 oms::ComponentFMUME::ComponentFMUME(const ComRef& cref, System* parentSystem, const std::string& fmuPath)
@@ -53,12 +53,6 @@ oms::ComponentFMUME::~ComponentFMUME()
   fmi2_import_destroy_dllfmu(fmu);
   fmi2_import_free(fmu);
   fmi_import_free_context(context);
-
-  if (!tempDir.empty() && boost::filesystem::is_directory(tempDir))
-  {
-    boost::filesystem::remove_all(tempDir);
-    logDebug("removed working directory: \"" + tempDir + "\"");
-  }
 }
 
 oms::Component* oms::ComponentFMUME::NewComponent(const oms::ComRef& cref, oms::System* parentSystem, const std::string& fmuPath)
@@ -75,14 +69,12 @@ oms::Component* oms::ComponentFMUME::NewComponent(const oms::ComRef& cref, oms::
     return NULL;
   }
 
-  boost::filesystem::path temp_root(parentSystem->getModel()->getTempDirectory());
-  boost::filesystem::path temp_temp = temp_root / "temp";
-  boost::filesystem::path temp_resources = temp_root / "resources";
+  filesystem::path temp_root(parentSystem->getModel()->getTempDirectory());
+  filesystem::path temp_temp = temp_root / "temp";
+  filesystem::path relFMUPath = parentSystem->copyResources() ? (filesystem::path("resources") / (parentSystem->getUniqueID() + "_" + std::string(cref) + ".fmu")) : filesystem::path(fmuPath);
+  filesystem::path absFMUPath = temp_root / relFMUPath;
 
-  boost::filesystem::path relFMUPath = boost::filesystem::path("resources") / (std::string(cref) + ".fmu");
-  boost::filesystem::path absFMUPath = temp_root / relFMUPath;
-
-  ComponentFMUME* component = new ComponentFMUME(cref, parentSystem, "resources/" + std::string(cref) + ".fmu");
+  ComponentFMUME* component = new ComponentFMUME(cref, parentSystem, relFMUPath.string());
 
   component->callbacks.malloc = malloc;
   component->callbacks.calloc = calloc;
@@ -92,13 +84,15 @@ oms::Component* oms::ComponentFMUME::NewComponent(const oms::ComRef& cref, oms::
   component->callbacks.log_level = jm_log_level_all;
   component->callbacks.context = 0;
 
+  // Copy the resource to the temp directory of the model? We don't want have
+  // do copy resources if importing an SSP or snapshot.
   if (parentSystem->copyResources())
-    boost::filesystem::copy_file(boost::filesystem::path(fmuPath), absFMUPath, boost::filesystem::copy_option::overwrite_if_exists);
+    oms_copy_file(filesystem::path(fmuPath), absFMUPath);
 
   // set temp directory
-  boost::filesystem::path tempDir = temp_temp / std::string(cref);
-  component->tempDir = tempDir.string();
-  if (!boost::filesystem::is_directory(tempDir) && !boost::filesystem::create_directory(tempDir))
+  filesystem::path tempDir = temp_temp / relFMUPath.stem();
+  component->setTempDir(tempDir.string());
+  if (!filesystem::is_directory(tempDir) && !filesystem::create_directory(tempDir))
   {
     logError("Creating temp directory for component \"" + std::string(cref) + "\" failed");
     return NULL;
@@ -107,7 +101,7 @@ oms::Component* oms::ComponentFMUME::NewComponent(const oms::ComRef& cref, oms::
   component->context = fmi_import_allocate_context(&component->callbacks);
 
   // check version of FMU
-  fmi_version_enu_t version = fmi_import_get_fmi_version(component->context, absFMUPath.string().c_str(), component->tempDir.c_str());
+  fmi_version_enu_t version = fmi_import_get_fmi_version(component->context, absFMUPath.string().c_str(), component->getTempDir().c_str());
   if (fmi_version_2_0_enu != version)
   {
     logError("Unsupported FMI version: " + std::string(fmi_version_to_string(version)));
@@ -116,7 +110,7 @@ oms::Component* oms::ComponentFMUME::NewComponent(const oms::ComRef& cref, oms::
   }
 
   // parse modelDescription.xml
-  component->fmu = fmi2_import_parse_xml(component->context, component->tempDir.c_str(), 0);
+  component->fmu = fmi2_import_parse_xml(component->context, component->getTempDir().c_str(), 0);
   if (!component->fmu)
   {
     logError("Error parsing modelDescription.xml");
@@ -483,6 +477,9 @@ oms_status_enu_t oms::ComponentFMUME::instantiate()
 
 oms_status_enu_t oms::ComponentFMUME::doEventIteration()
 {
+  const int maxIterations = Flags::MaxEventIteration();
+  int iterations = 0;
+
   CallClock callClock(clock);
   fmi2_status_t fmistatus;
   eventInfo.newDiscreteStatesNeeded = fmi2_true;
@@ -491,6 +488,9 @@ oms_status_enu_t oms::ComponentFMUME::doEventIteration()
   {
     fmistatus = fmi2_import_new_discrete_states(fmu, &eventInfo);
     if (fmi2_status_ok != fmistatus) return logError_FMUCall("fmi2_import_new_discrete_states", this);
+
+    if (++iterations >= maxIterations)
+      return logError("Event iteration reached max number of iterations (" + std::to_string(maxIterations) + ") for FMU " + std::string(getCref()));
   }
   return oms_status_ok;
 }
@@ -579,7 +579,7 @@ oms_status_enu_t oms::ComponentFMUME::getBoolean(const ComRef& cref, bool& value
   }
 
   if (!fmu || j < 0)
-    return oms_status_error;
+    return logError_UnknownSignal(getFullCref() + cref);
 
   fmi2_value_reference_t vr = allVariables[j].getValueReference();
   return getBoolean(vr, value);
@@ -609,12 +609,23 @@ oms_status_enu_t oms::ComponentFMUME::getInteger(const ComRef& cref, int& value)
   }
 
   if (!fmu || j < 0)
-    return oms_status_error;
+    return logError_UnknownSignal(getFullCref() + cref);
 
   fmi2_value_reference_t vr = allVariables[j].getValueReference();
   return getInteger(vr, value);
 }
 
+oms::Variable* oms::ComponentFMUME::getVariable(const ComRef& cref)
+{
+  CallClock callClock(clock);
+  for (size_t i=0; i < allVariables.size(); i++)
+    if (allVariables[i].getCref() == cref)
+      return &allVariables[i];
+
+  logError_UnknownSignal(getFullCref() + cref);
+  return NULL;
+}
+
 oms_status_enu_t oms::ComponentFMUME::getReal(const fmi2_value_reference_t& vr, double& value)
 {
   CallClock callClock(clock);
@@ -627,6 +638,29 @@ oms_status_enu_t oms::ComponentFMUME::getReal(const fmi2_value_reference_t& vr,
   if (std::isinf(value))
     return logError("getReal returned +/-inf");
 
+  // Check for FIB
+  auto block = fib.find(vr);
+  if (block != fib.end())
+  {
+    switch(block->second.faultType)
+    {
+      case oms_fault_type_bias:   // y = y.$original + faultValue
+        value += block->second.faultValue;
+        break;
+
+      case oms_fault_type_gain:   // y = y.$original * faultValue
+        value *= block->second.faultValue;
+        break;
+
+      case oms_fault_type_const:  // y = faultValue
+        value = block->second.faultValue;
+        break;
+
+      default:
+        return logError("Unhandled fault injection block");
+    }
+  }
+
   return oms_status_ok;
 }
 
@@ -644,7 +678,7 @@ oms_status_enu_t oms::ComponentFMUME::getReal(const ComRef& cref, double& value)
   }
 
   if (!fmu || j < 0)
-    return oms_status_error;
+    return logError_UnknownSignal(getFullCref() + cref);
 
   fmi2_value_reference_t vr = allVariables[j].getValueReference();
   return getReal(vr, value);
@@ -664,7 +698,7 @@ oms_status_enu_t oms::ComponentFMUME::setBoolean(const ComRef& cref, bool value)
   }
 
   if (!fmu || j < 0)
-    return oms_status_error;
+    return logError_UnknownSignal(getFullCref() + cref);
 
   if (oms_modelState_virgin == getModel()->getModelState())
     booleanStartValues[allVariables[j].getCref()] = value;
@@ -693,7 +727,7 @@ oms_status_enu_t oms::ComponentFMUME::setInteger(const ComRef& cref, int value)
   }
 
   if (!fmu || j < 0)
-    return oms_status_error;
+    return logError_UnknownSignal(getFullCref() + cref);
 
   if (oms_modelState_virgin == getModel()->getModelState())
     integerStartValues[allVariables[j].getCref()] = value;
@@ -721,7 +755,7 @@ oms_status_enu_t oms::ComponentFMUME::setReal(const ComRef& cref, double value)
   }
 
   if (!fmu || j < 0)
-    return oms_status_error;
+    return logError_UnknownSignal(getFullCref() + cref);
 
   if (getModel()->validState(oms_modelState_virgin|oms_modelState_enterInstantiation|oms_modelState_instantiated))
     if (allVariables[j].isCalculated() || allVariables[j].isIndependent())
@@ -743,7 +777,11 @@ oms_status_enu_t oms::ComponentFMUME::registerSignalsForResultFile(ResultWriter&
 {
   resultFileMapping.clear();
 
-  clock_id = resultFile.addSignal(std::string(getFullCref() + ComRef("$wallTime")), "wall-clock time [s]", SignalType_REAL);
+  if (Flags::WallTime())
+    clock_id = resultFile.addSignal(std::string(getFullCref() + ComRef("$wallTime")), "wall-clock time [s]", SignalType_REAL);
+  else
+    clock_id = 0;
+
   for (unsigned int i=0; i<allVariables.size(); ++i)
   {
     if (!exportVariables[i])
@@ -801,9 +839,13 @@ oms_status_enu_t oms::ComponentFMUME::registerSignalsForResultFile(ResultWriter&
 oms_status_enu_t oms::ComponentFMUME::updateSignals(ResultWriter& resultWriter)
 {
   CallClock callClock(clock);
-  SignalValue_t wallTime;
-  wallTime.realValue = clock.getElapsedWallTime();
-  resultWriter.updateSignal(clock_id, wallTime);
+
+  if (clock_id)
+  {
+    SignalValue_t wallTime;
+    wallTime.realValue = clock.getElapsedWallTime();
+    resultWriter.updateSignal(clock_id, wallTime);
+  }
 
   for (auto const &it : resultFileMapping)
   {
@@ -916,3 +958,26 @@ oms_status_enu_t oms::ComponentFMUME::removeSignalsFromResults(const char* regex
 
   return oms_status_ok;
 }
+
+oms_status_enu_t oms::ComponentFMUME::setFaultInjection(const oms::ComRef& signal, oms_fault_type_enu_t faultType, double faultValue)
+{
+  Variable* var = getVariable(signal);
+
+  if (!var || !var->isOutput())
+    return oms_status_error;
+
+  // remove fib?
+  if ((faultType == oms_fault_type_bias && faultValue == 0.0) ||
+      (faultType == oms_fault_type_gain && faultValue == 1.0))
+  {
+    fib.erase(var->getValueReference());
+  }
+  else
+  {
+    oms_fault_type_t ft;
+    ft.faultType = faultType;
+    ft.faultValue = faultValue;
+    fib[var->getValueReference()] = ft;
+  }
+  return oms_status_ok;
+}
