@@ -21,7 +21,7 @@
  * GNU version 3 is obtained from: http://www.gnu.org/copyleft/gpl.html.
  *
  * This program is distributed WITHOUT ANY WARRANTY; without
- * even the implied warranty of  MERCHANTABILITY or FITNESS
+ * even the implied warranty of MERCHANTABILITY or FITNESS
  * FOR A PARTICULAR PURPOSE, EXCEPT AS EXPRESSLY SET FORTH
  * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS OF OSMC-PL.
  *
@@ -36,6 +36,7 @@
 #include "ComponentFMUME.h"
 #include "ComponentTable.h"
 #include "ExternalModel.h"
+#include "Flags.h"
 #include "Model.h"
 #include "ResultWriter.h"
 #include "ssd/Tags.h"
@@ -43,7 +44,8 @@
 #include "SystemTLM.h"
 #include "SystemWC.h"
 #include "Types.h"
-#include <boost/filesystem.hpp>
+#include "Variable.h"
+#include <OMSFileSystem.h>
 #include <RegEx.h>
 
 oms::System::System(const oms::ComRef& cref, oms_system_enu_t type, oms::Model* parentModel, oms::System* parentSystem, oms_solver_enu_t solverMethod)
@@ -271,7 +273,7 @@ oms_status_enu_t oms::System::addSubModel(const oms::ComRef& cref, const std::st
     if (!validCref(cref))
       return logError_AlreadyInScope(getFullCref() + cref);
 
-    if (!boost::filesystem::exists(path))
+    if (!filesystem::exists(path))
       return logError("file does not exist: \"" + path + "\"");
 
     Component* component = NULL;
@@ -491,7 +493,7 @@ oms_status_enu_t oms::System::importFromSSD(const pugi::xml_node& node)
         connectors.back() = oms::Connector::NewConnector(*itConnectors);
         if (connectors.back())
         {
-          exportConnectors[getFullCref() + connectors.back()->getName()] = true;
+          exportConnectors[connectors.back()->getName()] = true;
           connectors.push_back(NULL);
         }
         else
@@ -948,7 +950,7 @@ oms_status_enu_t oms::System::addTLMConnection(const oms::ComRef& crefA, const o
 {
 #if !defined(NO_TLM)
   if (type != oms_system_tlm)
-    return logError_OnlyForTlmSystem;
+    return logError_OnlyForSystemTLM;
 
   oms::ComRef tailA(crefA);
   oms::ComRef headA = tailA.pop_front();
@@ -1182,7 +1184,7 @@ oms_status_enu_t oms::System::addExternalModel(const oms::ComRef& cref, std::str
 {
 #if !defined(NO_TLM)
   if (type != oms_system_tlm)
-    return logError_OnlyForTlmSystem;
+    return logError_OnlyForSystemTLM;
 
   if (!cref.isValidIdent())
     return oms_status_error;
@@ -1388,6 +1390,7 @@ oms_status_enu_t oms::System::delete_(const oms::ComRef& cref)
     auto component = components.find(front);
     if (component != components.end())
     {
+      logInfo("Delete " + std::string(front));
       deleteAllConectionsTo(front);
       delete component->second;
       components.erase(component);
@@ -1397,7 +1400,8 @@ oms_status_enu_t oms::System::delete_(const oms::ComRef& cref)
     for (int i=0; i<connectors.size()-1; ++i)
       if (connectors[i]->getName() == front)
       {
-        exportConnectors.erase(getFullCref() + front);
+        deleteAllConectionsTo(front);
+        exportConnectors.erase(front);
         delete connectors[i];
         connectors.pop_back();   // last element is always NULL
         connectors[i] = connectors.back();
@@ -1526,7 +1530,7 @@ oms_status_enu_t oms::System::getBoolean(const ComRef& cref, bool& value)
     }
   }
 
-  return oms_status_error;
+  return logError_UnknownSignal(getFullCref() + cref);
 }
 
 oms_status_enu_t oms::System::getInteger(const ComRef& cref, int& value)
@@ -1558,7 +1562,7 @@ oms_status_enu_t oms::System::getInteger(const ComRef& cref, int& value)
     }
   }
 
-  return oms_status_error;
+  return logError_UnknownSignal(getFullCref() + cref);
 }
 
 oms_status_enu_t oms::System::getReal(const ComRef& cref, double& value)
@@ -1590,7 +1594,28 @@ oms_status_enu_t oms::System::getReal(const ComRef& cref, double& value)
     }
   }
 
-  return oms_status_error;
+  return logError_UnknownSignal(getFullCref() + cref);
+}
+
+oms::Variable* oms::System::getVariable(const ComRef& cref)
+{
+  oms::ComRef tail(cref);
+  oms::ComRef head = tail.pop_front();
+
+  auto subsystem = subsystems.find(head);
+  if (subsystem != subsystems.end())
+    return subsystem->second->getVariable(tail);
+
+  auto component = components.find(head);
+  if (component != components.end())
+    return component->second->getVariable(tail);
+
+  //for (auto& connector : connectors)
+  //  if (connector && connector->getName() == cref)
+  //    return connector->getVariable();
+
+  logError_UnknownSignal(getFullCref() + cref);
+  return NULL;
 }
 
 oms_status_enu_t oms::System::setBoolean(const ComRef& cref, bool value)
@@ -1616,7 +1641,7 @@ oms_status_enu_t oms::System::setBoolean(const ComRef& cref, bool value)
       return oms_status_ok;
     }
 
-  return oms_status_error;
+  return logError_UnknownSignal(getFullCref() + cref);
 }
 
 oms_status_enu_t oms::System::setInteger(const ComRef& cref, int value)
@@ -1642,7 +1667,7 @@ oms_status_enu_t oms::System::setInteger(const ComRef& cref, int value)
       return oms_status_ok;
     }
 
-  return oms_status_error;
+  return logError_UnknownSignal(getFullCref() + cref);
 }
 
 oms_status_enu_t oms::System::setReal(const ComRef& cref, double value)
@@ -1668,7 +1693,7 @@ oms_status_enu_t oms::System::setReal(const ComRef& cref, double value)
       return oms_status_ok;
     }
 
-  return oms_status_error;
+  return logError_UnknownSignal(getFullCref() + cref);
 }
 
 oms_status_enu_t oms::System::getReals(const std::vector<oms::ComRef> &sr, std::vector<double> &values)
@@ -1720,7 +1745,10 @@ oms_status_enu_t oms::System::importFromSSD_ConnectionGeometry(const pugi::xml_n
 
 oms_status_enu_t oms::System::registerSignalsForResultFile(ResultWriter& resultFile)
 {
-  clock_id = resultFile.addSignal(std::string(getFullCref() + ComRef("$wallTime")), "wall-clock time [s]", SignalType_REAL);
+  if (Flags::WallTime())
+    clock_id = resultFile.addSignal(std::string(getFullCref() + ComRef("$wallTime")), "wall-clock time [s]", SignalType_REAL);
+  else
+    clock_id = 0;
 
   for (const auto& component : components)
     if (oms_status_ok != component.second->registerSignalsForResultFile(resultFile))
@@ -1762,9 +1790,12 @@ oms_status_enu_t oms::System::registerSignalsForResultFile(ResultWriter& resultF
 
 oms_status_enu_t oms::System::updateSignals(ResultWriter& resultFile)
 {
-  SignalValue_t wallTime;
-  wallTime.realValue = clock.getElapsedWallTime();
-  resultFile.updateSignal(clock_id, wallTime);
+  if (clock_id)
+  {
+    SignalValue_t wallTime;
+    wallTime.realValue = clock.getElapsedWallTime();
+    resultFile.updateSignal(clock_id, wallTime);
+  }
 
   for (const auto& component : components)
     if (oms_status_ok != component.second->updateSignals(resultFile))
@@ -1804,6 +1835,26 @@ oms_status_enu_t oms::System::updateSignals(ResultWriter& resultFile)
 
 oms_status_enu_t oms::System::addSignalsToResults(const char* regex)
 {
+  try
+  {
+    oms_regex exp(regex);
+    for (auto& x: exportConnectors)
+    {
+      if (x.second)
+        continue;
+
+      if (regex_match(std::string(x.first), exp))
+      {
+        //logInfo("added \"" + std::string(x.first) + "\" to results");
+        x.second = true;
+      }
+    }
+  }
+  catch (std::regex_error& e)
+  {
+    return logError("invalid regular expression");
+  }
+
   for (const auto& component : components)
     if (oms_status_ok != component.second->addSignalsToResults(regex))
       return oms_status_error;
@@ -1812,24 +1863,31 @@ oms_status_enu_t oms::System::addSignalsToResults(const char* regex)
     if (oms_status_ok != subsystem.second->addSignalsToResults(regex))
       return oms_status_error;
 
-  oms_regex exp(regex);
-  for (auto& x: exportConnectors)
-  {
-    if (x.second)
-      continue;
-
-    if (regex_match(std::string(x.first), exp))
-    {
-      //logInfo("added \"" + std::string(x.first) + "\" to results");
-      x.second = true;
-    }
-  }
-
   return oms_status_ok;
 }
 
 oms_status_enu_t oms::System::removeSignalsFromResults(const char* regex)
 {
+  try
+  {
+    oms_regex exp(regex);
+    for (auto& x: exportConnectors)
+    {
+      if (!x.second)
+        continue;
+
+      if (regex_match(std::string(x.first), exp))
+      {
+        //logInfo("removed \"" + std::string(x.first) + "\" from results");
+        x.second = false;
+      }
+    }
+  }
+  catch (std::regex_error& e)
+  {
+    return logError("invalid regular expression");
+  }
+
   for (const auto& component : components)
     if (oms_status_ok != component.second->removeSignalsFromResults(regex))
       return oms_status_error;
@@ -1838,18 +1896,44 @@ oms_status_enu_t oms::System::removeSignalsFromResults(const char* regex)
     if (oms_status_ok != subsystem.second->removeSignalsFromResults(regex))
       return oms_status_error;
 
-  oms_regex exp(regex);
-  for (auto& x: exportConnectors)
-  {
-    if (!x.second)
-      continue;
-
-    if (regex_match(std::string(x.first), exp))
-    {
-      //logInfo("removed \"" + std::string(x.first) + "\" from results");
-      x.second = false;
-    }
-  }
-
   return oms_status_ok;
 }
+
+bool oms::System::useThreadPool()
+{
+  return getModel()->useThreadPool();
+}
+
+ctpl::thread_pool& oms::System::getThreadPool()
+{
+  return getModel()->getThreadPool();
+}
+
+std::string oms::System::getUniqueID() const
+{
+  static unsigned int id = 0;
+  std::string str = std::to_string(++id);
+
+  while (str.length() < 4)
+    str = "0" + str;
+
+  return str;
+}
+
+oms_status_enu_t oms::System::setFaultInjection(const oms::ComRef& signal, oms_fault_type_enu_t faultType, double faultValue)
+{
+  oms::System* system = NULL;
+
+  oms::ComRef tail(signal);
+  oms::ComRef front = tail.pop_front();
+
+  auto subsystem = subsystems.find(front);
+  if (subsystem != subsystems.end())
+    return subsystem->second->setFaultInjection(tail, faultType, faultValue);
+
+  auto component = components.find(front);
+  if (component != components.end())
+    return component->second->setFaultInjection(tail, faultType, faultValue);
+
+  return oms_status_error;
+}
