@@ -21,7 +21,7 @@
  * GNU version 3 is obtained from: http://www.gnu.org/copyleft/gpl.html.
  *
  * This program is distributed WITHOUT ANY WARRANTY; without
- * even the implied warranty of  MERCHANTABILITY or FITNESS
+ * even the implied warranty of MERCHANTABILITY or FITNESS
  * FOR A PARTICULAR PURPOSE, EXCEPT AS EXPRESSLY SET FORTH
  * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS OF OSMC-PL.
  *
@@ -49,6 +49,12 @@
 #include <pugixml.hpp>
 #include <unordered_map>
 
+#if (BOOST_VERSION >= 105300)
+#include <ctpl.h>
+#else // use the standard queue
+#include <ctpl_stl.h>
+#endif
+
 namespace oms
 {
   class Component;
@@ -64,6 +70,7 @@ namespace oms
     System* getSystem(const ComRef& cref);
     Component* getComponent(const ComRef& cref);
     System* getSubSystem(const ComRef& cref);
+    Variable* getVariable(const ComRef& cref);
     const ComRef& getCref() const {return cref;}
     ComRef getFullCref() const;
     Element* getElement() {return &element;}
@@ -115,6 +122,7 @@ namespace oms
     const DirectedGraph& getInitialUnknownsGraph() {return initialUnknownsGraph;}
     const DirectedGraph& getOutputsGraph() {return outputsGraph;}
     oms_status_enu_t exportDependencyGraphs(const std::string& pathInitialization, const std::string& pathSimulation);
+    oms_status_enu_t setFaultInjection(const ComRef& signal, oms_fault_type_enu_t faultType, double faultValue);
 
     virtual oms_status_enu_t instantiate() = 0;
     virtual oms_status_enu_t initialize() = 0;
@@ -150,6 +158,11 @@ namespace oms
     double getMaximumStepSize() {return maximumStepSize;}
     oms_solver_enu_t getSolver() {return solverMethod;}
 
+    bool useThreadPool();
+    ctpl::thread_pool& getThreadPool();
+
+    std::string getUniqueID() const;
+
   protected:
     System(const ComRef& cref, oms_system_enu_t type, Model* parentModel, System* parentSystem, oms_solver_enu_t solverMethod);
 
@@ -170,6 +183,10 @@ namespace oms
     double minimumStepSize = 1e-4;
     double maximumStepSize = 1e-1;
     double initialStepSize = 1e-4;
+
+    std::unordered_map<unsigned int /*result file var ID*/, unsigned int /*allVariables ID*/> resultFileMapping;
+    std::unordered_map<ComRef, bool> exportConnectors;
+
   private:
     ComRef cref;
     oms_system_enu_t type;
@@ -191,9 +208,6 @@ namespace oms
 #endif
     std::vector<Connection*> connections;           ///< last element is always NULL
 
-    std::unordered_map<unsigned int /*result file var ID*/, unsigned int /*allVariables ID*/> resultFileMapping;
-    std::unordered_map<ComRef, bool> exportConnectors;
-
     oms_status_enu_t importFromSSD_ConnectionGeometry(const pugi::xml_node& node, const ComRef& crefA, const ComRef& crefB);
   };
 }
